@@ -5,7 +5,7 @@
 modname = 'saem_ref'
 distname = 'cubicweb-saem-ref'
 
-numversion = (0, 10, 1)
+numversion = (0, 12, 2)
 version = '.'.join(str(num) for num in numversion)
 
 license = 'LGPL'
@@ -14,18 +14,21 @@ author_email = 'contact@logilab.fr'
 description = "Référenciel de Système d'Archivage Électronique Mutualisé"
 web = 'http://www.cubicweb.org/project/%s' % distname
 
-__depends__ =  {'cubicweb': '>= 3.22.1',
+__depends__ =  {'cubicweb': '>= 3.23.0',
                 'cubicweb-squareui': None,
                 'cubicweb-addressbook': None,
-                'cubicweb-compound': '>= 0.2.1',
-                'cubicweb-relationwidget': '>= 0.3.2',
-                'cubicweb-skos': '>= 0.9.0',
+                'cubicweb-compound': '>= 0.3.0',
+                'cubicweb-oaipmh': None,
+                'cubicweb-relationwidget': '>= 0.3.3',
+                'cubicweb-skos': '>= 0.12.0',
                 'cubicweb-prov': None,
+                'cubicweb-pyramid': None,
                 'cubicweb-vtimeline': None,
                 'cubicweb-signedrequest': None,
                 'python-dateutil': None,
                 'pytz': None,
                 'psycopg2': None,
+                'rdflib': '>= 4.1',
                }
 __recommends__ = {}
 
@@ -55,7 +58,6 @@ data_files = [
 # check for possible extended cube layout
 for dname in ('entities', 'views', 'sobjects', 'hooks', 'schema', 'data',
               'wdoc', 'i18n', 'migration', 'lib', 'dataimport', 'scripts',
-              'backports',
               'migration/data'):
     if isdir(dname):
         data_files.append([join(THIS_CUBE_DIR, dname), listdir(dname)])
