@@ -1,4 +1,4 @@
-# copyright 2015 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# copyright 2015-2016 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 # contact http://www.logilab.fr -- mailto:contact@logilab.fr
 #
 # This program is free software: you can redistribute it and/or modify it under
@@ -31,12 +31,15 @@ from cubicweb.dataimport.importer import ExtEntity
 from cubes.skos import to_unicode
 from cubes.saem_ref import eactype_mapping, eacaddress_mapping, eacmaintenancetype_mapping
 
-ETYPES_ORDER_HINT = ('AgentKind', 'PhoneNumber', 'PostalAddress', 'Agent',
+TYPE_MAPPING = eactype_mapping.copy()
+TYPE_MAPPING['human'] = u'person'
+
+ETYPES_ORDER_HINT = ('AgentKind', 'PhoneNumber', 'PostalAddress', 'AuthorityRecord',
                      'AgentPlace', 'Mandate', 'LegalStatus', 'History',
-                     'Structure', 'AgentFunction', 'Occupation',
+                     'Structure', 'AgentFunction', 'Occupation', 'GeneralContext',
                      'AssociationRelation', 'ChronologicalRelation', 'HierarchicalRelation',
                      'EACResourceRelation', 'ExternalUri', 'EACSource',
-                     'Activity')
+                     'Agent', 'Activity')
 
 
 class InvalidEAC(RuntimeError):
@@ -70,6 +73,16 @@ def filter_none(func):
     return wraps(func)(wrapped)
 
 
+def filter_empty(func):
+    """Filter out empty ExtEntity (i.e. with empty ``values`` attribute)."""
+    @wraps(func)
+    def wrapper(self, *args, **kwargs):
+        for extentity in func(self, *args, **kwargs):
+            if extentity.values:
+                yield extentity
+    return wrapper
+
+
 def elem_maybe_none(func):
     """Method decorator for external entity builder function handling the case
     of `elem` being None.
@@ -89,6 +102,70 @@ def elem_maybe_none(func):
     return wraps(func)(wrapped)
 
 
+def add_xml_wrap_for(*etypes):
+    """Add an `xml_wrap` attribute in ExtEntity's values dictionnary."""
+    def decorator(func):
+        def wrapped(self, elem):
+            objectXMLWrap = self._elem_find(elem, 'eac:objectXMLWrap')
+            xmlwrap = None
+            if objectXMLWrap is not None:
+                nchildren = len(objectXMLWrap)
+                if nchildren >= 1:
+                    xmlwrap = objectXMLWrap[0]
+                if nchildren > 1:
+                    msg = self._('multiple children elements found in {0}').format(
+                        objectXMLWrap)
+                    self.import_log.record_warning(msg, line=objectXMLWrap.line)
+            attribute_added = False
+            for extentity in func(self, elem):
+                if xmlwrap is not None and extentity.etype in etypes:
+                    # prevent association of xmlwrap to several extentities.
+                    assert not attribute_added, 'xml_wrap attribute already added'
+                    extentity.values.setdefault('xml_wrap', set([])).add(
+                        etree.tostring(xmlwrap, encoding='utf-8'))
+                    attribute_added = True
+                yield extentity
+        return wraps(func)(wrapped)
+    return decorator
+
+
+def relate_to_record_through(etype, rtype):
+    """Add an ``rtype`` relationship from ``etype`` to the imported record."""
+    def decorator(func):
+        if inspect.isgeneratorfunction(func):
+            def wrapper(self, *args, **kwargs):
+                for extentity in func(self, *args, **kwargs):
+                    if extentity.etype == etype:
+                        extentity.values.setdefault(rtype, set()).add(self.record.extid)
+                    yield extentity
+        else:
+            def wrapper(self, *args, **kwargs):
+                extentity = func(self, *args, **kwargs)
+                if extentity and extentity.etype == etype:
+                    extentity.values.setdefault(rtype, set()).add(self.record.extid)
+                return extentity
+        return wraps(func)(wrapper)
+    return decorator
+
+
+def add_citations_for(etype):
+    """Handle import of citation tag for `etype` ExtEntity that is yielded by
+    decorated method.
+    """
+    def decorator(func):
+        @wraps(func)
+        def wrapper(self, elem):
+            for extentity in func(self, elem):
+                if extentity.etype == etype:
+                    for citation in self.build_citation(elem):
+                        extentity.values.setdefault(
+                            'has_citation', set()).add(citation.extid)
+                        yield citation
+                yield extentity
+        return wrapper
+    return decorator
+
+
 def require_tag(tagname):
     """Method decorator handling a mandatory tag within a XML element."""
     def warn(self, elem):
@@ -174,8 +251,8 @@ class EACCPFImporter(object):
     The importer will generate `extid`s using the `extid_generator` function
     if specified or use `uuid.uuid4` to generate unique `extid`s.
 
-    During import the `agent` attribute is set to the external entity of the
-    imported Agent.
+    During import the `record` attribute is set to the external entity of the
+    imported AuthorityRecord.
 
     Ref: http://eac.staatsbibliothek-berlin.de/fileadmin/user_upload/schema/cpfTagLibrary.html
     """
@@ -188,12 +265,13 @@ class EACCPFImporter(object):
         self._root = tree.getroot()
         self.namespaces = self._root.nsmap.copy()
         self.namespaces['eac'] = self.namespaces.pop(None)
+        self.namespaces.setdefault('xlink', 'http://www.w3.org/1999/xlink')
         self.import_log = import_log
         if extid_generator is None:
             def extid_generator():
                 return str(uuid4())
         self._gen_extid = extid_generator
-        self.agent = ExtEntity('Agent', None, {})
+        self.record = ExtEntity('AuthorityRecord', None, {})
         # Store a mapping of XML elements to produced ExtEntities
         self._visited = {}
 
@@ -253,13 +331,14 @@ class EACCPFImporter(object):
     def _elem_findall(self, *args):
         return self._elem_find(*args, method='findall')
 
+    @filter_empty
     def external_entities(self):
         """Parse a EAC XML file to and yield external entities."""
         # control element.
         control = self._elem_find(self._root, 'eac:control')
         for extentity in self.parse_control(control):
             yield extentity
-        # Agents (identity tags) are within cpfDescription tag.
+        # Records (identity tags) are within cpfDescription tag.
         cpf_desc = self._elem_find(self._root, 'eac:cpfDescription')
         if cpf_desc is None:
             raise MissingTag('cpfDescription')
@@ -277,55 +356,52 @@ class EACCPFImporter(object):
         # relations element.
         for extentity in self.parse_relations(cpf_desc):
             yield extentity
-        # Agent is complete.
-        self.record_visited(self._root, self.agent)
-        yield self.agent
+        # Record is complete.
+        self.record_visited(self._root, self.record)
+        yield self.record
 
     def parse_identity(self, identity):
         """Parse the `identity` tag and yield external entities, possibly
-        updating agent's `values` dict.
+        updating record's `values` dict.
         """
         # entityId
         isni = self._elem_find(identity, 'eac:entityId')
         if isni is not None and isni.text:
-            self.record_visited(isni, self.agent)
-            self.agent.values['isni'] = set([unicode(isni.text)])
+            self.record_visited(isni, self.record)
+            self.record.values['isni'] = set([unicode(isni.text)])
         # entityType
         akind = self._elem_find(identity, 'eac:entityType')
         if akind is None:
             raise MissingTag('entityType', 'identity')
         agent_kind = self.build_agent_kind(akind)
         yield agent_kind
-        self.agent.values['agent_kind'] = set([agent_kind.extid])
-        # nameEntry.
-        # look for the nameEntry with authorizedForm, take its part
-        # element.
-        name_entry_part = identity.xpath(
-            '//eac:nameEntry/eac:part[../eac:authorizedForm]',
-            namespaces=self.namespaces)
-        if name_entry_part is not None:
-            elem = name_entry_part[0]
-            self.record_visited(elem.getparent(), self.agent)
-            self.record_visited(elem, self.agent)
-            self.agent.values.setdefault('name', set()).add(unicode(elem.text))
+        self.record.values['agent_kind'] = set([agent_kind.extid])
+        for name_entry in self._elem_findall(identity, 'eac:nameEntry'):
+            self.record_visited(name_entry, self.record)
+            if self._elem_find(name_entry, 'eac:authorizedForm') is not None:
+                break
         else:
-            raise NotImplementedError('found no nameEntry with authorizedForm')
+            msg = self._('found no nameEntry tag within an authorizedForm '
+                         'element, using %s instead') % name_entry
+            self.import_log.record_info(msg, line=name_entry.sourceline)
+        part = self._elem_find(name_entry, 'eac:part')
+        self.record.values['name'] = set([unicode(part.text)])
 
     @filter_none
     def parse_description(self, description):
         """Parse the `description` tag and yield external entities, possibly
-        updating agent's `values` dict.
+        updating record's `values` dict.
         """
         # dates.
         daterange = description.xpath('eac:existDates/eac:dateRange',
                                       namespaces=self.namespaces)
         if daterange:
             elem = daterange[0]
-            self.record_visited(elem, self.agent)
-            self.record_visited(elem.getparent(), self.agent)
+            self.record_visited(elem, self.record)
+            self.record_visited(elem.getparent(), self.record)
             dates = self.parse_daterange(elem)
             if dates:
-                self.agent.values.update(dates)
+                self.record.values.update(dates)
         # address.
         for place in self.find_nested(description, 'eac:place', 'eac:places'):
             for extentity in self.build_place(place):
@@ -340,7 +416,8 @@ class EACCPFImporter(object):
                 yield extentity
         # history
         for history in self._elem_findall(description, 'eac:biogHist'):
-            yield self.build_history(history)
+            for extentity in self.build_history(history):
+                yield extentity
         # structure
         for structure in self._elem_findall(description, 'eac:structureOrGenealogy'):
             yield self.build_structure(structure)
@@ -353,6 +430,10 @@ class EACCPFImporter(object):
                                            'eac:occupations'):
             for extentity in self.build_occupation(occupation):
                 yield extentity
+        # general context
+        for context in self._elem_findall(description, 'eac:generalContext'):
+            for extentity in self.build_generalcontext(context):
+                yield extentity
 
     def find_nested(self, elem, tagname, innertag):
         """Return a list of element with `tagname` within `element` possibly
@@ -393,8 +474,9 @@ class EACCPFImporter(object):
         else:
             ptag = '{%(eac)s}p' % self.namespaces
             desc = '\n'.join(etree.tostring(child, encoding=unicode,
-                                            method='html')
-                             for child in elem.iterchildren(ptag))
+                                            method='html').strip()
+                             for child in elem.iterchildren(ptag)
+                             if len(child) != 0 or child.text)
             if desc:
                 desc_format = u'text/html'
             else:
@@ -408,13 +490,14 @@ class EACCPFImporter(object):
     def build_agent_kind(self, elem):
         """Build a AgentKind external entity"""
         # Map EAC entity types to our terminolgy.
-        type_mapping = eactype_mapping.copy()
-        type_mapping['human'] = u'person'
-        kind = type_mapping.get(elem.text, elem.text)
+        kind = TYPE_MAPPING.get(elem.text, elem.text)
         agentkind_id = 'agentkind/' + kind
-        return ExtEntity('AgentKind', agentkind_id, {'name': set([kind])})
+        return ExtEntity('AgentKind', agentkind_id, {'name': set([unicode(kind)])})
 
     @elem_maybe_none
+    @relate_to_record_through('LegalStatus', 'legal_status_agent')
+    @filter_empty
+    @add_citations_for('LegalStatus')
     @equivalent_concept('eac:term', 'LegalStatus')
     def build_legal_status(self, elem, **kwargs):
         """Build a `LegalStatus` external entity.
@@ -430,15 +513,15 @@ class EACCPFImporter(object):
         dates = self.parse_daterange(dateelem)
         if dates:
             values.update(dates)
-        # don't build an entity if it only owns the agent relation
-        if values:
-            values['legal_status_agent'] = set([self.agent.extid])
-            extentity = ExtEntity('LegalStatus', self._gen_extid(), values)
-            if dateelem is not None:
-                self.record_visited(dateelem, extentity)
-            yield extentity
+        legalstatus = ExtEntity('LegalStatus', self._gen_extid(), values)
+        if dateelem is not None:
+            self.record_visited(dateelem, legalstatus)
+        yield legalstatus
 
     @elem_maybe_none
+    @relate_to_record_through('Mandate', 'mandate_agent')
+    @filter_empty
+    @add_citations_for('Mandate')
     @equivalent_concept('eac:term', 'Mandate')
     def build_mandate(self, elem, **kwargs):
         """Build a `Mandate` external entity.
@@ -454,57 +537,57 @@ class EACCPFImporter(object):
         dates = self.parse_daterange(dateelem)
         if dates:
             values.update(dates)
-        citation = self.build_citation(self._elem_find(elem, 'eac:citation'))
-        if citation:
-            values['has_citation'] = set([citation.extid])
-            yield citation
-        if values:
-            values['mandate_agent'] = set([self.agent.extid])
-            yield ExtEntity('Mandate', self._gen_extid(), values)
+        yield ExtEntity('Mandate', self._gen_extid(), values)
 
     @elem_maybe_none
     def build_citation(self, elem):
         """Build a `Citation` external entity."""
-        note = elem.text.strip() if elem.text else u''
-        uri = elem.attrib.get('{%(xlink)s}href' % self.namespaces)
-        if not note and not uri:
-            msg = self._('element {0} has no text nor (valid) link').format(
-                etree.tostring(elem))
-            self.import_log.record_warning(msg, line=elem.sourceline)
-            return
-        values = {}
-        if uri:
-            values['uri'] = set([unicode(uri)])
-        if note:
-            values['note'] = set([unicode(note)])
-            if u'<span>' in note:
-                values['note_format'] = set([u'text/html'])
-        return ExtEntity('Citation', self._gen_extid(), values)
+        for citation_elem in self._elem_findall(elem, "eac:citation"):
+            note = citation_elem.text.strip() if citation_elem.text else u''
+            uri = citation_elem.attrib.get('{%(xlink)s}href' % self.namespaces)
+            if not note and not uri:
+                msg = self._('element {0} has no text nor (valid) link').format(
+                    etree.tostring(citation_elem))
+                self.import_log.record_warning(msg, line=citation_elem.sourceline)
+                return
+            values = {}
+            if uri:
+                values['uri'] = set([unicode(uri)])
+            if note:
+                values['note'] = set([unicode(note)])
+                if u'<span>' in note:
+                    values['note_format'] = set([u'text/html'])
+            yield ExtEntity('Citation', self._gen_extid(), values)
 
+    @relate_to_record_through('History', 'history_agent')
+    @filter_empty
+    @add_citations_for('History')
     @elem_maybe_none
     def build_history(self, elem):
         """Build a `History` external entity."""
         desc, desc_format = self.parse_tag_content(elem)
+        values = {}
         if desc:
-            values = {'history_agent': set([self.agent.extid]),
-                      'text': set([desc]),
-                      'text_format': set([desc_format])}
-            return ExtEntity('History', self._gen_extid(), values)
+            values.update(text=set([desc]), text_format=set([desc_format]))
+        yield ExtEntity('History', self._gen_extid(), values)
 
     @elem_maybe_none
+    @relate_to_record_through('Structure', 'structure_agent')
     def build_structure(self, elem):
         """Build a `Structure` external entity."""
         desc, desc_format = self.parse_tag_content(elem)
         if desc:
-            values = {'structure_agent': set([self.agent.extid]),
-                      'description': set([desc]),
+            values = {'description': set([desc]),
                       'description_format': set([desc_format])}
             return ExtEntity('Structure', self._gen_extid(), values)
 
+    @relate_to_record_through('AgentPlace', 'place_agent')
+    @filter_empty
+    @add_citations_for('AgentPlace')
     @equivalent_concept('eac:placeEntry', 'AgentPlace')
     def build_place(self, elem):
         """Build a AgentPlace external entity"""
-        values = {'place_agent': set([self.agent.extid])}
+        values = {}
         role = self._elem_find(elem, 'eac:placeRole')
         if role is not None:
             values['role'] = set([unicode(role.text)])
@@ -528,6 +611,9 @@ class EACCPFImporter(object):
                         unicode(line.text))
         yield ExtEntity('PostalAddress', self._gen_extid(), address_entity)
 
+    @relate_to_record_through('AgentFunction', 'function_agent')
+    @filter_empty
+    @add_citations_for('AgentFunction')
     @equivalent_concept('eac:term', 'AgentFunction')
     def build_function(self, elem):
         """Build a `AgentFunction`s external entities"""
@@ -536,10 +622,11 @@ class EACCPFImporter(object):
         if term is not None:
             values['name'] = set([unicode(term.text)])
         values.update(self.parse_tag_description(elem))
-        if values:
-            values['function_agent'] = set([self.agent.extid])
-            yield ExtEntity('AgentFunction', self._gen_extid(), values)
+        yield ExtEntity('AgentFunction', self._gen_extid(), values)
 
+    @relate_to_record_through('Occupation', 'occupation_agent')
+    @filter_empty
+    @add_citations_for('Occupation')
     @equivalent_concept('eac:term', 'Occupation')
     def build_occupation(self, elem):
         """Build a `Occupation`s external entities"""
@@ -552,13 +639,21 @@ class EACCPFImporter(object):
         dates = self.parse_daterange(dateelem)
         if dates:
             values.update(dates)
-        citation = self.build_citation(self._elem_find(elem, 'eac:citation'))
-        if citation:
-            values['has_citation'] = set([citation.extid])
-            yield citation
-        if values:
-            values['occupation_agent'] = set([self.agent.extid])
-            yield ExtEntity('Occupation', self._gen_extid(), values)
+        yield ExtEntity('Occupation', self._gen_extid(), values)
+
+    @relate_to_record_through('GeneralContext', 'general_context_of')
+    @filter_empty
+    @add_citations_for('GeneralContext')
+    def build_generalcontext(self, elem):
+        """Build a `GeneralContext` external entity"""
+        values = {}
+        content, content_format = self.parse_tag_content(elem)
+        if content:
+            values.update({
+                'content': set([content]),
+                'content_format': set([content_format]),
+            })
+        yield ExtEntity('GeneralContext', self._gen_extid(), values)
 
     @elem_maybe_none
     def parse_daterange(self, elem):
@@ -613,7 +708,7 @@ class EACCPFImporter(object):
 
     def parse_relations(self, cpf_description):
         """Parse the `relations` tag and yield external entities, possibly
-        updating agent's `values` dict.
+        updating record's `values` dict.
         """
         relations = self._elem_find(cpf_description, 'eac:relations')
         if relations is None:
@@ -627,14 +722,17 @@ class EACCPFImporter(object):
             for extentity in self.build_resource_relation(rrel):
                 yield extentity
 
+    @add_xml_wrap_for('AssociationRelation', 'ChronologicalRelation',
+                      'HierarchicalRelation')
     def build_relation(self, elem):
-        """Build a relation between agents external entity (with proper type)."""
+        """Build a relation between records external entity (with proper type)."""
         relationship = elem.attrib.get('cpfRelationType')
         if relationship is None:
             self.import_log.record_warning(self._(
-                'found no cpfRelationType attribute in element %s') % etree.tostring(elem),
+                'found no cpfRelationType attribute in element %s, defaulting '
+                'to associative') % etree.tostring(elem),
                 line=elem.sourceline)
-            return
+            relationship = 'associative'
         try:
             # "other_role" (resp. "agent_role") role designates the object of the relation (resp.
             # the agent described in the EAC-CPF instance).
@@ -671,11 +769,12 @@ class EACCPFImporter(object):
             return
         rentry = self._elem_find(elem, 'eac:relationEntry')
         if rentry is not None and rentry.text.strip():
-            yield ExtEntity('Agent', obj_uri, {'name': set([unicode(rentry.text)]),
-                                               'agent_kind': set(['agentkind/unknown-agent-kind'])})
+            yield ExtEntity('AuthorityRecord', obj_uri,
+                            {'name': set([unicode(rentry.text)]),
+                             'agent_kind': set(['agentkind/unknown-agent-kind'])})
         else:
             yield external_uri(obj_uri)
-        values = {agent_role: set([self.agent.extid]), other_role: set([obj_uri])}
+        values = {agent_role: set([self.record.extid]), other_role: set([obj_uri])}
         dates = self.parse_daterange(
             self._elem_find(elem, 'eac:dateRange'))
         if dates:
@@ -689,6 +788,7 @@ class EACCPFImporter(object):
         values.update(self.parse_tag_description(elem))
         yield ExtEntity(etype, self._gen_extid(), values)
 
+    @add_xml_wrap_for('EACResourceRelation')
     def build_resource_relation(self, elem):
         """Build a `EACResourceRelation` external entity (along with
         ExternalUri entities).
@@ -702,7 +802,7 @@ class EACCPFImporter(object):
         yield external_uri(obj_uri)
         values = {
             'resource_relation_resource': set([obj_uri]),
-            'resource_relation_agent': set([self.agent.extid]),
+            'resource_relation_agent': set([self.record.extid]),
         }
         resource_role = elem.attrib.get('{%(xlink)s}role' % self.namespaces)
         if resource_role:
@@ -721,16 +821,17 @@ class EACCPFImporter(object):
         """Parse the `control` tag."""
         record_id = self._elem_find(control, 'eac:recordId')
         if record_id is not None and record_id.text and record_id.text.strip():
-            self.agent.extid = record_id.text.strip()
-            self.record_visited(record_id, self.agent)
+            self.record.extid = record_id.text.strip()
+            self.record_visited(record_id, self.record)
         else:
-            self.agent.extid = self._gen_extid()
+            self.record.extid = self._gen_extid()
             self.import_log.record_warning(self._(
                 'found no recordId element in control tag, using %s as cwuri') %
-                self.agent.extid)
+                self.record.extid)
         for elem in control.xpath('eac:sources/eac:source',
                                   namespaces=self.namespaces):
-            yield self.build_source(elem)
+            for extentity in self.build_source(elem):
+                yield extentity
         for elem in control.xpath('eac:maintenanceHistory/eac:maintenanceEvent',
                                   namespaces=self.namespaces):
             for extentity in self.build_maintenance_event(elem):
@@ -738,9 +839,9 @@ class EACCPFImporter(object):
 
     def build_maintenance_event(self, elem):
         """Parse a `maintenanceEvent` tag, yielding a prov:Activity external
-        entity along with necessary Agents.
+        entity along with necessary Records.
         """
-        values = {'generated': set([self.agent.extid])}
+        values = {'generated': set([self.record.extid])}
         event_type = self.parse_event_type(self._elem_find(elem, 'eac:eventType'))
         if event_type is not None:
             values['type'] = set([event_type])
@@ -760,18 +861,10 @@ class EACCPFImporter(object):
         values.update(self.parse_tag_description(elem, 'eac:eventDescription'))
         agent = self._elem_find(elem, 'eac:agent')
         if agent is not None and agent.text:
-            agent_type = self._elem_find(elem, 'eac:agentType')
-            agent_kind = self.build_agent_kind(agent_type)
-            if agent_kind is not None:
-                yield agent_kind
-                agent_kind_extid = agent_kind.extid
-            else:
-                agent_kind_extid = 'agentkind/unknown-agent-kind'
-            agent_id = self._gen_extid()
-            yield ExtEntity('Agent', agent_id,
-                            {'name': set([unicode(agent.text)]),
-                             'agent_kind': set([agent_kind_extid])})
-            values['associated_with'] = set([agent_id])
+            record_id = self._gen_extid()
+            yield ExtEntity('Agent', record_id,
+                            {'name': set([unicode(agent.text)])})
+            values['associated_with'] = set([record_id])
         yield ExtEntity('Activity', self._gen_extid(), values)
 
     @elem_maybe_none
@@ -793,6 +886,9 @@ class EACCPFImporter(object):
                 return None
             return event_type
 
+    @relate_to_record_through('EACSource', 'source_agent')
+    @filter_empty
+    @add_xml_wrap_for('EACSource')
     def build_source(self, elem):
         """Parse a `source` tag, yielding EACSource external entities.
         """
@@ -804,6 +900,4 @@ class EACCPFImporter(object):
         if entry is not None and entry.text:
             values['title'] = set([unicode(entry.text)])
         values.update(self.parse_tag_description(elem))
-        if values:
-            values['source_agent'] = set([self.agent.extid])
-            return ExtEntity('EACSource', self._gen_extid(), values)
+        yield ExtEntity('EACSource', self._gen_extid(), values)
