@@ -15,6 +15,7 @@
 # You should have received a copy of the GNU Lesser General Public License along
 # with this program. If not, see <http://www.gnu.org/licenses/>.
 """cubicweb-saem-ref entity's classes for exporting agents to eac"""
+from functools import wraps
 
 from lxml import etree
 
@@ -26,9 +27,44 @@ from cubes.saem_ref import (cwuri_url, eactype_mapping, eacaddress_mapping,
 from cubes.saem_ref.entities import AbstractXmlAdapter
 
 
-class AgentEACAdapter(AbstractXmlAdapter):
+def add_object_xml_wrap(func):
+    """Add an objectXMLWrap sub-element to the element returned by `func`."""
+    def wrapper(self, entity, *args, **kwargs):
+        elem = func(self, entity, *args, **kwargs)
+        if entity.xml_wrap is not None:
+            objectXMLWrap = self.element('objectXMLWrap', parent=elem)
+            objectXMLWrap.append(etree.parse(entity.xml_wrap).getroot())
+        return elem
+    return wraps(func)(wrapper)
+
+
+def add_descriptive_note(func):
+    @wraps(func)
+    def wrapper(self, entity, *args, **kwargs):
+        element = func(self, entity, *args, **kwargs)
+        if element is not None and entity.description:
+            self.element('descriptiveNote', parent=element).extend(
+                self._eac_richstring_paragraph_elements(entity, "description"))
+        return element
+    return wrapper
+
+
+def add_citation(func):
+    @wraps(func)
+    def wrapper(self, entity, *args, **kwargs):
+        element = func(self, entity, *args, **kwargs)
+        for citation in entity.has_citation:
+            attrs = {'xlink:type': 'simple'}
+            if citation.uri:
+                attrs['xlink:href'] = citation.uri
+            self.element('citation', parent=element, attributes=attrs, text=citation.note)
+        return element
+    return wrapper
+
+
+class AuthorityRecordEACAdapter(AbstractXmlAdapter):
     __regid__ = 'EAC-CPF'
-    __select__ = is_instance('Agent')
+    __select__ = is_instance('AuthorityRecord')
 
     namespaces = {
         None: u'urn:isbn:1-931666-33-4',
@@ -40,7 +76,11 @@ class AgentEACAdapter(AbstractXmlAdapter):
     @property
     def file_name(self):
         """Return a file name for the dump."""
-        return u'{0}.xml'.format(self.entity.dc_title())
+        if self.entity.ark:
+            name = self.entity.ark.replace("/", "_")
+        else:
+            name = unicode(self.entity.eid)
+        return u'EAC_{0}.xml'.format(name)
 
     def dump(self):
         """Return an XML string representing the given agent using the EAC-CPF schema."""
@@ -59,7 +99,7 @@ class AgentEACAdapter(AbstractXmlAdapter):
 
     def control_element(self, eac_cpf_elt):
         control_elt = self.element('control', parent=eac_cpf_elt)
-        self.element('recordId', parent=control_elt, text=self.entity.ark)
+        self.element('recordId', parent=control_elt, text=unicode(self.entity.eid))
         self.maintenance_status_element(control_elt)
         self.publication_status_element(control_elt)
         self.maintenance_agency_element(control_elt)
@@ -104,7 +144,7 @@ class AgentEACAdapter(AbstractXmlAdapter):
     def sources_element(self, control_elt):
         sources_elt = self.element('sources')
         for eac_source in self.entity.reverse_source_agent:
-            self.source_element(eac_source, sources_elt)
+            sources_elt.append(self.source_element(eac_source))
         if len(sources_elt):
             control_elt.append(sources_elt)
 
@@ -121,18 +161,30 @@ class AgentEACAdapter(AbstractXmlAdapter):
         agent = self.entity
         description_elt = self.element('description', parent=cpfdescription_elt)
         self.exist_dates_element(description_elt)
-        for place in agent.reverse_place_agent:
-            self.place_element(place, description_elt)
-        for function in agent.reverse_function_agent:
-            self.function_element(function, description_elt)
-        for legal_status in agent.reverse_legal_status_agent:
-            self.legal_status_element(legal_status, description_elt)
-        for mandate in agent.reverse_mandate_agent:
-            self.mandate_element(mandate, description_elt)
+        for relation, group_tagname, get_element in [
+            ('reverse_place_agent', 'places', self.place_element),
+            ('reverse_function_agent', 'functions', self.function_element),
+            ('reverse_legal_status_agent', 'legalStatuses', self.legal_status_element),
+            ('reverse_occupation_agent', 'occupations', self.occupation_element),
+            ('reverse_mandate_agent', 'mandates', self.mandate_element),
+        ]:
+            relateds = getattr(agent, relation)
+            if relateds:
+                group_elt = self.element(group_tagname, parent=description_elt)
+                for related in relateds:
+                    group_elt.append(get_element(related))
         for structure in agent.reverse_structure_agent:
-            self.structure_element(structure, description_elt)
+            structure_elt = self.structure_element(structure)
+            if len(structure_elt):
+                description_elt.append(structure_elt)
+        for generalcontext in agent.reverse_general_context_of:
+            generalcontext_elt = self.generalcontext_element(generalcontext)
+            if len(generalcontext_elt):
+                description_elt.append(generalcontext_elt)
         for history in agent.reverse_history_agent:
-            self.bioghist_element(history, description_elt)
+            bioghist_elt = self.bioghist_element(history)
+            if len(bioghist_elt):
+                description_elt.append(bioghist_elt)
 
     def relations_element(self, cpfdescription_elt):
         relations_elt = self.element('relations')
@@ -140,15 +192,16 @@ class AgentEACAdapter(AbstractXmlAdapter):
             ('reverse_hierarchical_child', 'hierarchical_parent', 'hierarchical-parent'),
             ('reverse_hierarchical_parent', 'hierarchical_child', 'hierarchical-child'),
             ('reverse_chronological_successor', 'chronological_predecessor', 'temporal-earlier'),
-            ('reverse_chronological_predecessor', 'chronological_successor', 'temporal-after'),
+            ('reverse_chronological_predecessor', 'chronological_successor', 'temporal-later'),
             ('reverse_association_to', 'association_from', 'associative'),
             ('reverse_association_from', 'association_to', 'associative')
         ]:
             for relation in getattr(self.entity, rtype):
-                self.cpfrelation_element(relation, relations_elt, cw_rtype=rel_rtype,
-                                         eac_rtype=eac_rtype)
+                relations_elt.append(
+                    self.cpfrelation_element(relation, rel_rtype, eac_rtype))
         for resource_relation in self.entity.reverse_resource_relation_agent:
-            self.resource_relation_element(resource_relation, relations_elt)
+            relations_elt.append(
+                self.resource_relation_element(resource_relation))
         if len(relations_elt):
             cpfdescription_elt.append(relations_elt)
 
@@ -170,13 +223,14 @@ class AgentEACAdapter(AbstractXmlAdapter):
             event_elt.append(self.element('agent'))
         self._elt_text_from_attr('eventDescription', activity, 'description', parent=event_elt)
 
-    def source_element(self, eac_source, sources_elt):
+    @add_descriptive_note
+    @add_object_xml_wrap
+    def source_element(self, eac_source):
         url = eac_source.url
         attributes = {'xlink:href': url, 'xlink:type': 'simple'} if url else None
-        source_elt = self.element('source', attributes=attributes, parent=sources_elt)
+        source_elt = self.element('source', attributes=attributes)
         self.element('sourceEntry', parent=source_elt, text=eac_source.title)
-        self._eac_inner_paragraph_element('descriptiveNote', eac_source.description,
-                                          parent=source_elt)
+        return source_elt
 
     def exist_dates_element(self, description_elt):
         date_range = self._eac_date_range_xml_elt(self.entity.start_date, self.entity.end_date)
@@ -184,71 +238,103 @@ class AgentEACAdapter(AbstractXmlAdapter):
             exist_dates = self.element('existDates', parent=description_elt)
             exist_dates.append(date_range)
 
-    def place_element(self, place, description_elt):
-        place_elt = self.element('place', parent=description_elt)
+    @add_citation
+    def place_element(self, place):
+        place_elt = self.element('place')
         for attr, eac_name in [('role', 'placeRole'), ('name', 'placeEntry')]:
             eac_elt = self._elt_text_from_attr(eac_name, place, attr, parent=place_elt)
             if eac_elt is not None and eac_name == 'placeEntry' and place.equivalent_concept:
                 eac_elt.attrib['vocabularySource'] = cwuri_url(place.equivalent_concept[0])
         for address in place.place_address:
             self.address_element(address, place_elt)
+        return place_elt
 
-    def function_element(self, function, description_elt):
-        function_elt = self.element('function', parent=description_elt)
+    @add_descriptive_note
+    @add_citation
+    def function_element(self, function):
+        function_elt = self.element('function')
         term_elt = self._elt_text_from_attr('term', function, 'name', parent=function_elt)
         if term_elt is not None and function.equivalent_concept:
             term_elt.attrib['vocabularySource'] = cwuri_url(function.equivalent_concept[0])
-        self._eac_inner_paragraph_element('descriptiveNote', function.description,
-                                          parent=function_elt)
+        return function_elt
 
-    def legal_status_element(self, legal_status, description_elt):
-        legal_status_elt = self.element('legalStatus', parent=description_elt)
+    @add_descriptive_note
+    @add_citation
+    def legal_status_element(self, legal_status):
+        legal_status_elt = self.element('legalStatus')
         self._elt_text_from_attr('term', legal_status, 'term', parent=legal_status_elt)
-        self._eac_inner_paragraph_element('descriptiveNote', legal_status.description,
-                                          parent=legal_status_elt)
+        return legal_status_elt
 
-    def mandate_element(self, mandate, description_elt):
-        mandate_elt = self.element('mandate', parent=description_elt)
+    @add_descriptive_note
+    @add_citation
+    def occupation_element(self, occupation):
+        occupation_elt = self.element('occupation')
+        term_elt = self._elt_text_from_attr('term', occupation, 'term', parent=occupation_elt)
+        if term_elt is not None and occupation.equivalent_concept:
+            term_elt.attrib['vocabularySource'] = cwuri_url(occupation.equivalent_concept[0])
+        self._eac_date_range_xml_elt(occupation.start_date, occupation.end_date,
+                                     parent=occupation_elt)
+        return occupation_elt
+
+    @add_descriptive_note
+    @add_citation
+    def mandate_element(self, mandate):
+        mandate_elt = self.element('mandate')
         term_elt = self._elt_text_from_attr('term', mandate, 'term', parent=mandate_elt)
         if term_elt is not None and mandate.equivalent_concept:
             term_elt.attrib['vocabularySource'] = cwuri_url(mandate.equivalent_concept[0])
-        self._eac_inner_paragraph_element('descriptiveNote', mandate.description,
-                                          parent=mandate_elt)
+        return mandate_elt
 
-    def structure_element(self, structure, description_elt):
-        self._eac_inner_paragraph_element('structureOrGenealogy', structure.description,
-                                          parent=description_elt)
+    def structure_element(self, structure):
+        structure_elt = self.element('structureOrGenealogy')
+        if structure.description:
+            structure_elt.extend(self._eac_richstring_paragraph_elements(structure, "description"))
+        return structure_elt
 
-    def bioghist_element(self, history, description_elt):
-        self._eac_inner_paragraph_element('biogHist', history.text, parent=description_elt)
+    @add_citation
+    def generalcontext_element(self, context):
+        context_elt = self.element('generalContext')
+        if context.content:
+            context_elt.extend(self._eac_richstring_paragraph_elements(context, 'content'))
+        return context_elt
 
-    def cpfrelation_element(self, relation, relations_elt, cw_rtype=None,
-                            eac_rtype='associative'):
-        related = getattr(relation, cw_rtype, [None])[0]
-        assert related is not None
+    @add_citation
+    def bioghist_element(self, history):
+        bioghist_elt = self.element('biogHist')
+        if history.text:
+            bioghist_elt.extend(self._eac_richstring_paragraph_elements(history, "text"))
+        return bioghist_elt
+
+    @add_descriptive_note
+    @add_object_xml_wrap
+    def cpfrelation_element(self, relation, cw_rtype, eac_rtype):
+        related = relation.related(cw_rtype).one()  # exactly one target (schema)
         relation_elt = self.element('cpfRelation',
                                     attributes={'cpfRelationType': eac_rtype,
                                                 'xlink:href': related.absolute_url(),
-                                                'xlink:type': 'simple'},
-                                    parent=relations_elt)
+                                                'xlink:type': 'simple'})
         if related.cw_etype != 'ExternalUri':
             self.element('relationEntry', parent=relation_elt, text=related.name)
         self._eac_date_range_xml_elt(getattr(relation, 'start_date', None),
                                      getattr(relation, 'end_date', None),
                                      parent=relation_elt)
-        self._eac_inner_paragraph_element('descriptiveNote', relation.description,
-                                          parent=relation_elt)
+        return relation_elt
 
-    def resource_relation_element(self, resource_relation, relations_elt):
+    @add_object_xml_wrap
+    def resource_relation_element(self, resource_relation):
         resource = resource_relation.resource_relation_resource[0]
-        res_rel_elt = self.element('resourceRelation',
-                                   attributes={'resourceRelationType': resource_relation.agent_role,
-                                               'xlink:href': resource.uri,
-                                               'xlink:role': resource_relation.resource_role,
-                                               'xlink:type': 'simple'},
-                                   parent=relations_elt)
+        attrs = {
+            'xlink:href': resource.uri,
+            'xlink:type': 'simple',
+        }
+        if resource_relation.resource_role:
+            attrs['xlink:role'] = resource_relation.resource_role
+        if resource_relation.agent_role:
+            attrs['resourceRelationType'] = resource_relation.agent_role
+        res_rel_elt = self.element('resourceRelation', attributes=attrs)
         self._eac_date_range_xml_elt(resource_relation.start_date, resource_relation.end_date,
                                      parent=res_rel_elt)
+        return res_rel_elt
 
     def agent_element(self, agent, maintenance_event_elt):
         self.element('agentType', maintenance_event_elt, text='human')
@@ -292,9 +378,15 @@ class AgentEACAdapter(AbstractXmlAdapter):
                          attributes={'standardDate': dt.isoformat()}, text=dt.isoformat())
         return date_range
 
-    def _eac_inner_paragraph_element(self, tag, text, parent):
-        """Return an EAC lxml ``Element`` having the given tag and enclosing a ``<p>`` element with
-        the given text."""
-        if text:
-            descr_elt = self.element(tag, parent=parent)
-            self.element('p', parent=descr_elt, text=text)
+    def _eac_richstring_paragraph_elements(self, entity, attr_name):
+        fmt = getattr(entity, attr_name + "_format")
+        if fmt == "text/plain":
+            value = getattr(entity, attr_name)
+            if not value:
+                return []
+            return [self.element('p', text=value)]
+        else:
+            value = entity.printable_value(attr_name)
+            if not value:
+                return []
+            return list(etree.fromstring(u"<root>{0}</root>".format(value)))
