@@ -18,13 +18,15 @@
 
 from lxml import etree
 
-from cubicweb.entities import AnyEntity
+from cubicweb.uilib import cut
+from cubicweb.entities import AnyEntity, fetch_config
 from cubicweb.predicates import is_instance, is_in_state
 
 from cubes.compound.entities import IClonableAdapter
+from cubes.oaipmh.entities import OAISetSpec
 
 from cubes.saem_ref import cwuri_url
-from cubes.saem_ref.entities import oai, AbstractXmlAdapter
+from cubes.saem_ref.entities import AbstractXmlAdapter, oai
 
 _ = unicode
 
@@ -44,6 +46,8 @@ class SEDAProfileIClonableAdapter(IClonableAdapter):
 
 class SEDAProfile(AnyEntity):
     __regid__ = 'SEDAProfile'
+    fetch_attrs, cw_fetch_order = fetch_config(('title', 'ark',
+                                                'description', 'description_format'))
 
     @property
     def archives(self):
@@ -75,7 +79,7 @@ class SEDAProfile(AnyEntity):
         return None
 
 
-class TransferringAgentOAISetSpec(oai.OAISetSpec):
+class TransferringAgentOAISetSpec(OAISetSpec):
     """OAI-PMH set specifier to match SEDAProfile related to a transferring
     agent.
     """
@@ -109,6 +113,7 @@ class SEDAProfileOAIPMHRecordAdapter(oai.OAIPMHActiveRecordAdapter):
 
 class ProfileArchiveObject(AnyEntity):
     __regid__ = 'ProfileArchiveObject'
+    fetch_attrs, cw_fetch_order = fetch_config(('seda_name',))
 
     def dc_title(self):
         """Return the dc_title for an entity supporting seda_name relation."""
@@ -148,6 +153,7 @@ class ProfileArchiveObject(AnyEntity):
 
 class ProfileDocument(AnyEntity):
     __regid__ = 'ProfileDocument'
+    fetch_attrs, cw_fetch_order = fetch_config(('user_annotation', 'seda_document_type_code'))
 
     @property
     def description(self):
@@ -173,7 +179,11 @@ class ProfileDocument(AnyEntity):
 
     def dc_title(self):
         dtype = self.document_type_code.seda_document_type_code_value[0].dc_title()
-        return u'%s #%s' % (dtype, self.eid)
+        if self.user_annotation:
+            title = u'%s (%s)' % (cut(self.user_annotation, 100), dtype)
+        else:
+            title = u'%s #%s' % (dtype, self.eid)
+        return title
 
 
 def _seda_profile_element_dc_title(entity):
@@ -183,6 +193,7 @@ def _seda_profile_element_dc_title(entity):
 
 class SEDAContentDescription(AnyEntity):
     __regid__ = 'SEDAContentDescription'
+    fetch_attrs, cw_fetch_order = fetch_config(('seda_description_level',))
 
     @property
     def description(self):
@@ -211,6 +222,8 @@ class SEDADescriptionLevel(AnyEntity):
 
 class SEDAAppraisalRule(AnyEntity):
     __regid__ = 'SEDAAppraisalRule'
+    fetch_attrs, cw_fetch_order = fetch_config(('seda_appraisal_rule_code',
+                                                'seda_appraisal_rule_duration'))
 
     def dc_title(self):
         return _seda_profile_element_dc_title(self)
@@ -219,6 +232,10 @@ class SEDAAppraisalRule(AnyEntity):
     def appraisal_rule_code(self):
         return self.seda_appraisal_rule_code[0]
 
+    @property
+    def appraisal_rule_duration(self):
+        return self.seda_appraisal_rule_duration[0]
+
 
 class SEDAAccessRestrictionCode(AnyEntity):
     __regid__ = 'SEDAAccessRestrictionCode'
@@ -229,6 +246,7 @@ class SEDAAccessRestrictionCode(AnyEntity):
 
 class SEDAName(AnyEntity):
     __regid__ = 'SEDAName'
+    fetch_attrs, cw_fetch_order = fetch_config(('value',))
 
     def dc_title(self):
         return self.printable_value('value') or self._cw._('<no value>')
@@ -236,6 +254,7 @@ class SEDAName(AnyEntity):
 
 class SEDADescription(AnyEntity):
     __regid__ = 'SEDADescription'
+    fetch_attrs, cw_fetch_order = fetch_config(('value',))
 
     def dc_title(self):
         return self.printable_value('value') or self._cw._('<no value>')
@@ -243,6 +262,7 @@ class SEDADescription(AnyEntity):
 
 class SEDADate(AnyEntity):
     __regid__ = 'SEDADate'
+    fetch_attrs, cw_fetch_order = fetch_config(('value',))
 
     def dc_title(self):
         return self.printable_value('value') or self._cw._('<no value>')
@@ -312,6 +332,12 @@ class SEDA1XSDExport(AbstractXmlAdapter):
         'xsd': 'http://www.w3.org/2001/XMLSchema',
         'qdt': 'fr:gouv:culture:archivesdefrance:seda:v1.0:QualifiedDataType:1',
         'udt': 'urn:un:unece:uncefact:data:standard:UnqualifiedDataType:10',
+        'clmDAFFileTypeCode': 'urn:un:unece:uncefact:codelist:draft:DAF:fileTypeCode:2009-08-18',
+        'clmIANACharacterSetCode':
+        'urn:un:unece:uncefact:codelist:standard:IANA:CharacterSetCode:2007-05-14',
+        'clmIANAMIMEMediaType':
+        'urn:un:unece:uncefact:codelist:standard:IANA:MIMEMediaType:2008-11-12',
+        'clm60133': 'urn:un:unece:uncefact:codelist:standard:6:0133:40106',
     }
     root_attributes = {
         'attributeFormDefault': 'unqualified',
@@ -360,7 +386,7 @@ class SEDA1XSDExport(AbstractXmlAdapter):
 
     def xsd_archive(self, parent, archive):
         """Append XSD elements for an archive to the given parent node."""
-        archive_node = self.xsd_element(parent, 'Archive', 'ArchiveType',
+        archive_node = self.xsd_element(parent, 'Archive',
                                         cardinality=archive.user_cardinality,
                                         annotation=archive.user_annotation,
                                         xsd_attributes=[XSDAttr('Id', 'xsd:ID')])
@@ -382,7 +408,7 @@ class SEDA1XSDExport(AbstractXmlAdapter):
 
     def xsd_archive_object(self, parent, archive_object):
         """Append XSD elements for the archive object to the given parent node."""
-        ao_node = self.xsd_element(parent, self.archive_object_tag_name, 'ArchiveObjectType',
+        ao_node = self.xsd_element(parent, self.archive_object_tag_name,
                                    cardinality=archive_object.user_cardinality,
                                    annotation=archive_object.user_annotation,
                                    xsd_attributes=[XSDAttr('Id', 'xsd:ID')])
@@ -401,7 +427,7 @@ class SEDA1XSDExport(AbstractXmlAdapter):
 
     def xsd_document(self, parent, document):
         """Append XSD elements for the document to the given parent node."""
-        document_node = self.xsd_element(parent, 'Document', 'DocumentType',
+        document_node = self.xsd_element(parent, 'Document',
                                          cardinality=document.user_cardinality,
                                          annotation=document.user_annotation,
                                          xsd_attributes=[XSDAttr('Id', 'xsd:ID')])
@@ -438,7 +464,7 @@ class SEDA1XSDExport(AbstractXmlAdapter):
             self.xsd_description(document_node, document.description)
 
     def xsd_appraisal_rule(self, parent, appraisal_rule):
-        ar_node = self.xsd_element(parent, 'Appraisal', 'AppraisalRuleType',
+        ar_node = self.xsd_element(parent, 'Appraisal',
                                    cardinality=appraisal_rule.user_cardinality,
                                    annotation=appraisal_rule.user_annotation,
                                    xsd_attributes=[XSDAttr('Id', 'xsd:ID')])
@@ -449,8 +475,9 @@ class SEDA1XSDExport(AbstractXmlAdapter):
                          annotation=ar_code.user_annotation,
                          xsd_attributes=[LIST_VERSION_ID_2009])
         ar_duration = appraisal_rule.seda_appraisal_rule_duration[0]
+        ar_duration_value = _concept_value(ar_duration, 'seda_appraisal_rule_duration_value')
         self.xsd_element(ar_node, 'Duration', 'qdt:ArchivesDurationType',
-                         value=ar_duration.value,
+                         value=ar_duration_value,
                          annotation=ar_duration.user_annotation)
         # hard-coded start date
         self.xsd_element(ar_node, 'StartDate', 'udt:DateType')
@@ -460,7 +487,6 @@ class SEDA1XSDExport(AbstractXmlAdapter):
     def xsd_access_restriction(self, parent, access_restriction):
         """Append XSD elements for an access restriction to the given parent node."""
         ar_node = self.xsd_element(parent, self.access_restriction_tag_name,
-                                   'AccessRestrictionRulesType',
                                    cardinality=access_restriction.user_cardinality,
                                    annotation=access_restriction.user_annotation,
                                    xsd_attributes=[XSDAttr('Id', 'xsd:ID')])
@@ -473,7 +499,7 @@ class SEDA1XSDExport(AbstractXmlAdapter):
 
     def xsd_content_description(self, parent, content_description):
         """Append XSD elements for a description content to the given parent node"""
-        cd_node = self.xsd_element(parent, 'ContentDescription', 'ContentDescriptionType',
+        cd_node = self.xsd_element(parent, 'ContentDescription',
                                    cardinality=content_description.user_cardinality,
                                    annotation=content_description.user_annotation,
                                    xsd_attributes=[XSDAttr('Id', 'xsd:ID')])
@@ -516,7 +542,7 @@ class SEDA1XSDExport(AbstractXmlAdapter):
 
     def xsd_keyword(self, parent, keyword):
         """Append XSD elements for a keyword to the given parent node"""
-        kw_node = self.xsd_element(parent, self.kw_tag_name, 'KeywordType',
+        kw_node = self.xsd_element(parent, self.kw_tag_name,
                                    cardinality=keyword.user_cardinality,
                                    annotation=keyword.user_annotation,
                                    xsd_attributes=[XSDAttr('Id', 'xsd:ID')])
@@ -595,10 +621,10 @@ class SEDA1XSDExport(AbstractXmlAdapter):
         """
         assert name, 'cannot build a XSD element tag with an empty name %r' % name
         attributes = {'name': name}
-        if xs_type is not None:
-            attributes['type'] = xs_type
         if value is not None:
             attributes['fixed'] = unicode(value)
+        if xs_type is not None and not xsd_attributes:
+            attributes['type'] = xs_type
         assert cardinality in ('0..1', '0..n', '1', '1..n')
         if cardinality != '1':
             if cardinality[0] == '0':
@@ -607,10 +633,19 @@ class SEDA1XSDExport(AbstractXmlAdapter):
                 attributes['maxOccurs'] = 'unbounded'
         element = self.element('xsd:element', parent, attributes)
         if annotation:
-            self.element('xsd:annotation', element).text = annotation
+            annotation_node = self.element('xsd:annotation', element)
+            self.element('xsd:documentation', annotation_node).text = annotation
+        children_parent = None
+        if xs_type is None:
+            attributes_parent = self.element('xsd:complexType', element)
+            children_parent = self.element('xsd:sequence', attributes_parent)
+        elif xsd_attributes:
+            ct = self.element('xsd:complexType', element)
+            scontent = self.element('xsd:simpleContent', ct)
+            attributes_parent = self.element('xsd:extension', scontent, {'base': xs_type})
         for xsd_attr in xsd_attributes:
-            self.element('xsd:attribute', element, xsd_attr)
-        return element
+            self.element('xsd:attribute', attributes_parent, xsd_attr)
+        return children_parent
 
     # compatibility testing ########################################################################
 
@@ -660,22 +695,18 @@ class SEDA02XSDExport(SEDA1XSDExport):
     """Adapter to build an XSD representation of a SEDA profile, using SEDA 0.2 specification"""
     __regid__ = 'SEDA-0.2.xsd'
 
-    namespaces = {
-        None: 'fr:gouv:ae:archive:draft:standard_echange_v0.2',
-        'xsd': 'http://www.w3.org/2001/XMLSchema',
-        'qdt': 'fr:gouv:ae:archive:draft:standard_echange_v0.2:QualifiedDataType:1',
-        'udt': 'urn:un:unece:uncefact:data:standard:UnqualifiedDataType:6',
-    }
-    root_attributes = {
-        'attributeFormDefault': 'unqualified',
-        'elementFormDefault': 'qualified',
-        'targetNamespace': 'fr:gouv:ae:archive:draft:standard_echange_v0.2',
-        'version': '1.1',
-    }
+    namespaces = SEDA1XSDExport.namespaces.copy()
+    namespaces[None] = 'fr:gouv:ae:archive:draft:standard_echange_v0.2'
+    namespaces['qdt'] = 'fr:gouv:ae:archive:draft:standard_echange_v0.2:QualifiedDataType:1'
+    namespaces['udt'] = 'urn:un:unece:uncefact:data:standard:UnqualifiedDataType:6'
+
+    root_attributes = SEDA1XSDExport.root_attributes.copy()
+    root_attributes['targetNamespace'] = 'fr:gouv:ae:archive:draft:standard_echange_v0.2'
+    root_attributes['version'] = '1.1'
 
     def xsd_archive(self, parent, archive):
         """Append XSD elements for an archive to the given parent node."""
-        archive_node = self.xsd_element(parent, 'Contains', 'ArchiveType',
+        archive_node = self.xsd_element(parent, 'Contains',
                                         cardinality=archive.user_cardinality,
                                         annotation=archive.user_annotation,
                                         xsd_attributes=[XSDAttr('Id', 'xsd:ID')])
@@ -699,7 +730,7 @@ class SEDA02XSDExport(SEDA1XSDExport):
 
     def xsd_content_description(self, parent, content_description):
         """Append XSD elements for a description content to the given parent node"""
-        cd_node = self.xsd_element(parent, 'ContentDescription', 'ContentDescriptionType',
+        cd_node = self.xsd_element(parent, 'ContentDescription',
                                    cardinality=content_description.user_cardinality,
                                    annotation=content_description.user_annotation,
                                    xsd_attributes=[XSDAttr('Id', 'xsd:ID')])
