@@ -22,9 +22,9 @@ from logilab.common.registry import objectify_predicate
 from yams import ValidationError
 from yams.schema import role_name
 
-from cubicweb import NoResultError
 from cubicweb.schema import META_RTYPES
-from cubicweb.predicates import is_instance, adaptable, on_fire_transition
+from cubicweb.predicates import (is_instance, adaptable, on_fire_transition,
+                                 relation_possible)
 from cubicweb.server import hook
 from cubicweb.hooks import metadata
 
@@ -102,6 +102,7 @@ class AddOrRemoveChildrenHook(hook.Hook):
     __regid__ = 'compound.graph.updated'
     __select__ = hook.Hook.__select__ & contained_relation()
     events = ('before_add_relation', 'before_delete_relation')
+    category = 'metadata'
 
     def __call__(self):
         for relationship, role, target in qualify_relation(self._cw.entity_from_eid(self.eidfrom),
@@ -118,6 +119,7 @@ class UpdateEntityHook(hook.Hook):
     __regid__ = 'compound.graph.updated'
     __select__ = hook.Hook.__select__ & adaptable('IContainer', 'IContained')
     events = ('before_update_entity',)
+    category = 'metadata'
 
     def __call__(self):
         UpdateModificationDateOp.get_instance(self._cw).add_entity(self.entity)
@@ -159,35 +161,43 @@ class UpdateModificationDateOp(hook.DataOperationMixIn, hook.Operation):
                 safety_belt.add(entity.eid)
 
 
-# Transformation of ExternalUri to Agent  ######################################
+# Transformation of ExternalUri to AuthorityRecord
 
-class ReplaceExternalUriByAgentHook(ReplaceExternalUriByEntityHook):
-    """Replace ExternalUri by an Agent"""
-    __select__ = ReplaceExternalUriByEntityHook.__select__ & is_instance('Agent')
+class ReplaceExternalUriByAuthorityRecordHook(ReplaceExternalUriByEntityHook):
+    """Replace ExternalUri by an AuthorityRecord"""
+    __select__ = ReplaceExternalUriByEntityHook.__select__ & is_instance('AuthorityRecord')
 
 
 # ARK generation ###############################################################
 
-
-class AssignARKToSKOSHook(hook.Hook):
-    """when an agent, SEDA profile, scheme or concept is created, assign a local ark to it"""
+class AssignARKHook(hook.Hook):
+    """When an entity supporting ARK is created, assign it a local ark to."""
     __regid__ = 'saem_ref.ark.assign'
-    __select__ = hook.Hook.__select__ & is_instance('Agent', 'SEDAProfile',
-                                                    'ConceptScheme', 'Concept')
+    __select__ = hook.Hook.__select__ & relation_possible('ark')
     events = ('before_add_entity',)
     order = metadata.InitMetaAttrsHook.order - 1
 
     def __call__(self):
-        set_skos_ark_and_cwuri(self._cw, self.entity.eid, self.entity.cw_edited)
+        if not self.entity.cw_edited.get('ark'):
+            naa_what = self.entity.cw_adapt_to('IArkNAALocator').naa_what()
+        else:
+            naa_what = None
+        set_ark_and_cwuri(self._cw, self.entity.eid, self.entity.cw_etype,
+                          self.entity.cw_edited, naa_what=naa_what)
 
 
-def set_skos_ark_and_cwuri(cw, eid, entity_attrs):
+def set_ark_and_cwuri(cw, eid, etype, entity_attrs, naa_what=None):
     ark = entity_attrs.get('ark')
     if not ark:
         cwuri = entity_attrs.get('cwuri')
         ark = None if cwuri is None else extract_ark(cwuri)
         if ark is None:
-            generator = cw.vreg['adapters'].select('IARKGenerator', cw, eid=eid)
+            if naa_what is None:
+                msg = _('an ARK identifier has to be generated but no Name Assigning Authority is '
+                        'specified')
+                raise ValidationError(None, {'': msg})
+            generator = cw.vreg['adapters'].select('IARKGenerator', cw, eid=eid, etype=etype,
+                                                   naa_what=naa_what)
             ark = generator.generate_ark()
         entity_attrs['ark'] = ark
     if 'cwuri' not in entity_attrs:
@@ -214,29 +224,6 @@ def extract_ark(url):
     return ark
 
 
-# Agent for CWUser #############################################################
-
-class CreateAgentForCWUser(hook.Hook):
-    """Create an Agent for each CWUser"""
-    __regid__ = 'saem_ref.create-agent-for-cwuser'
-    __select__ = hook.Hook.__select__ & is_instance('CWUser')
-    events = ('after_add_entity', )
-
-    def __call__(self):
-        try:
-            akind = self._cw.find('AgentKind', name=u'person').one()
-        except NoResultError:
-            if self.entity.login in ('admin', 'anon'):
-                # during instance creation
-                return
-            self.error('no agent kind "person", cannot create an Agent for %s',
-                       self.entity)
-            return
-        self._cw.create_entity('Agent', name=self.entity.login,
-                               agent_kind=akind,
-                               agent_user=self.entity)
-
-
 # Life-cycle logging ###########################################################
 
 class Record(object):
@@ -348,6 +335,7 @@ class LogContainerCreationHook(hook.Hook):
     __select__ = hook.Hook.__select__ & adaptable('IContainer')
     __regid__ = 'saem_ref.log.add'
     events = ('after_add_entity',)
+    category = 'prov.logging'
 
     def __call__(self):
         activity = Record([('created', self.entity.cw_etype)])
@@ -361,6 +349,7 @@ class LogModificationHook(hook.Hook):
     __select__ = hook.Hook.__select__ & adaptable('IContainer', 'IContained')
     __regid__ = 'saem_ref.log.update'
     events = ('after_update_entity',)
+    category = 'prov.logging'
 
     def __call__(self):
         entity = self.entity
@@ -392,6 +381,7 @@ class LogRelationHook(hook.Hook):
     __select__ = hook.Hook.__select__ & contained_relation()
     __regid__ = 'saem_ref.log.relation'
     events = ('after_add_relation', 'before_delete_relation')
+    category = 'prov.logging'
 
     def __call__(self):
         for relationship, role, target in qualify_relation(self._cw.entity_from_eid(self.eidfrom),
@@ -510,75 +500,26 @@ class DepreciateProfileOnReplacePublishedHook(hook.Hook):
 
 # Relations deposit agent - archival agent  ####################################
 
-class CheckArchivalAgentRelOperation(hook.DataOperationMixIn, hook.Operation):
-    def precommit_event(self):
-        for agent in self.get_data():
-            if agent.has_role(u'deposit'):
-                if not agent.archival_agent:
-                    errors = {role_name('archival_agent', 'subject'):
-                              _('this relation is mandatory on deposit agents')}
-                    raise ValidationError(agent.eid, errors)
-
-
-class CheckArchivalAgentRelOnAddAgentHook(hook.Hook):
-    """When creating an agent of type "deposit", it must have an archival_agent
-    relation
-    """
-    __select__ = hook.Hook.__select__ & is_instance('Agent')
-    __regid__ = 'saem_ref.check_archival_agent_relation_exists_add_agent'
-    events = ('after_add_entity', )
-
-    def __call__(self):
-        CheckArchivalAgentRelOperation.get_instance(self._cw).add_data(self.entity)
-
-
-class CheckArchivalAgentRelHookOnDeleteArchivalAgent(hook.Hook):
-    """When deleting a "archival_agent" on an agent, make sure this one is not a deposit agent,
-    or be linked to a new archival agent
-    """
-    __select__ = hook.Hook.__select__ & hook.match_rtype('archival_agent')
-    __regid__ = 'saem_ref.check_archival_agent_relation_exists_del_archival_agent'
-    events = ('before_delete_relation', )
-
-    def __call__(self):
-        CheckArchivalAgentRelOperation.get_instance(self._cw).add_data(
-            self._cw.entity_from_eid(self.eidfrom))
-
-
-class CheckArchivalAgentOnArchivalRoleUpdateHook(hook.Hook):
-    """When adding a "deposit" role to an agent, make sure this agent is linked to an archival agent
-    """
-    __select__ = hook.Hook.__select__ & hook.match_rtype('archival_role')
-    __regid__ = 'saem_ref.archival_role.add'
-    events = ('after_add_relation', )
-
-    def __call__(self):
-        if self._cw.entity_from_eid(self.eidto).name == u'deposit':
-            CheckArchivalAgentRelOperation.get_instance(self._cw).add_data(
-                self._cw.entity_from_eid(self.eidfrom))
-
-
-class DontDeleteAgentIfArchival(hook.Hook):
+class DontDeleteUnitIfArchival(hook.Hook):
     """Before deleting an agent, if it is an archival agent, make sure it isn't
-    associated to another agent through the "archival_agent" relation.
+    associated to an organization through the "archival_unit" relation.
     """
     __select__ = hook.Hook.__select__ & hook.match_rtype('archival_role')
     __regid__ = 'saem_ref.archival_role.delete'
     events = ('before_delete_relation', )
 
     def __call__(self):
-        agent = self._cw.entity_from_eid(self.eidfrom)
+        ou = self._cw.entity_from_eid(self.eidfrom)
         role = self._cw.entity_from_eid(self.eidto)
-        if role.name == u'archival' and agent.reverse_archival_agent:
+        if role.name == u'archival' and ou.reverse_archival_unit:
             errors = {role_name('archival_role', 'subject'):
-                      _('this agent is the archival agent of other agents, '
+                      _('this organization unit is the archival unit of some organization, '
                         'therefore the role "archival" cannot be deleted')}
-            raise ValidationError(agent.eid, errors)
+            raise ValidationError(ou.eid, errors)
 
 
 # ensure that equivalent_concept Concept has vocabulary_source defined ####################
 
-
 class EnsureVocabularySource(hook.Hook):
     """When a equivalent_concept relation is set and targets a Concept, ensure that the
     vocabulary_source relation is set to the concept's scheme. This should not be necessary when
