@@ -16,14 +16,14 @@
 # with this program. If not, see <http://www.gnu.org/licenses/>.
 """cubicweb-saem-ref schema"""
 
-from yams.buildobjs import (EntityType, RelationDefinition, String, Date,
+from yams.buildobjs import (EntityType, RelationDefinition, String, Date, Bytes, Int,
                             RichString, ComputedRelation, DEFAULT_RELPERMS)
 from yams.constraints import UniqueConstraint, BoundaryConstraint, Attribute
 
 from cubicweb.schema import (RO_ATTR_PERMS, ERQLExpression, RRQLExpression,
                              RQLConstraint, RQLVocabularyConstraint, WorkflowableEntityType,
                              make_workflowable)
-from cubicweb.schemas.base import ExternalUri
+from cubicweb.schemas.base import ExternalUri, EmailAddress
 
 from cubes.skos.schema import ConceptScheme
 from cubes.prov.schema import Agent
@@ -45,6 +45,13 @@ def dated_entity_type(cls):
     return cls
 
 
+def xml_wrap(cls):
+    """Class decorator adding an `xml_wrap` attribute to an EntityType."""
+    desc = _('XML elements not contained in EAC-CPF namespace')
+    cls.add_relation(Bytes(description=desc), name='xml_wrap')
+    return cls
+
+
 def seda_profile_element(cardinalities=None, default_cardinality=None):
     """Class decorator adding attributes to configure a SEDA field.
     """
@@ -200,42 +207,206 @@ ExternalUri.__permissions__ = ExternalUri.__permissions__.copy()
 ExternalUri.__permissions__['update'] = ()
 
 
+# Customization of EmailAddress entity type.
+EmailAddress.remove_relation('alias')
+
+
 # Customization of Agent entity type.
 make_workflowable(Agent)
 groups_permissions(Agent)
 Agent.add_relation(String(required=True, fulltextindexed=True), name='name')
-Agent.add_relation(String(unique=True,
-                          description=_('International Standard Name Identifier')),
-                   name='isni')
-dated_entity_type(Agent)
 
 
+@dated_entity_type
+@groups_permissions
+class AuthorityRecord(WorkflowableEntityType):
+    name = String(required=True, fulltextindexed=True)
+    isni = String(unique=True,
+                  description=_('International Standard Name Identifier'))
+
 # Customization of skos schema.
 make_workflowable(ConceptScheme)
 publication_permissions(ConceptScheme)
 
 
+class Organization(EntityType):
+    __permissions__ = {
+        'read': ('managers', 'users', 'guests'),
+        'add': ('managers', ),
+        'update': ('managers', ),
+        'delete': ('managers', ),
+    }
+    name = String(required=True, fulltextindexed=True, unique=True)
+
+
+class OrganizationUnit(WorkflowableEntityType):
+    __permissions__ = {
+        'read': ('managers', 'users', 'guests'),
+        'add': ('managers', ),
+        'update': ('managers', ),
+        'delete': ('managers', ),
+    }
+    __unique_together__ = [('name', 'authority')]
+    name = String(required=True, fulltextindexed=True, unique=True)
+
+
+class user_authority(RelationDefinition):
+    __permissions__ = {
+        'read': ('managers', 'users', 'guests'),
+        'add': ('managers',),
+        'delete': (),
+    }
+    name = 'authority'
+    subject = 'CWUser'
+    object = 'Organization'
+    cardinality = '?*'
+    inlined = True
+    constraints = [
+        RQLConstraint('NOT EXISTS(A agent_user S) '
+                      'OR EXISTS(B agent_user S, B authority O)'),
+    ]
+
+
+class others_authority(RelationDefinition):
+    __permissions__ = {
+        'read': ('managers', 'users', 'guests'),
+        'add': ('managers', RRQLExpression('U authority O'),),
+        'delete': (),
+    }
+    name = 'authority'
+    subject = ('OrganizationUnit', 'Agent')
+    object = 'Organization'
+    cardinality = '1*'
+    composite = 'object'
+    inlined = True
+
+
 class agent_user(RelationDefinition):
+    __permissions__ = {
+        'read': ('managers', 'users', 'guests'),
+        'add': ('managers',),
+        'delete': ('managers',),
+    }
     subject = 'Agent'
     object = 'CWUser'
     cardinality = '??'
     inlined = True
     description = _('the application user related to this agent')
+    constraints = [
+        RQLConstraint('NOT EXISTS(O authority A) '
+                      'OR EXISTS(O authority B, S authority B)'),
+    ]
+
+
+class _authority_record(RelationDefinition):
+    __permissions__ = {
+        'read': ('managers', 'users', 'guests'),
+        'add': ('managers',),
+        'delete': ('managers',),
+    }
+    name = 'authority_record'
+    object = 'AuthorityRecord'
+    cardinality = '??'
+    inlined = True
+    description = _('the authority record describing this agent')
+
+
+class ou_authority_record(_authority_record):
+    subject = 'OrganizationUnit'
+    description = _('the authority record describing this organization unit')
+    constraints = [
+        RQLConstraint('O agent_kind K, K name "authority"'),
+    ]
+
+
+class o_authority_record(ou_authority_record):
+    subject = 'Organization'
+    description = _('the authority record describing this organization')
+
+
+class agent_authority_record(_authority_record):
+    subject = 'Agent'
+    description = _('the authority record describing this agent')
+    constraints = [
+        RQLConstraint('O agent_kind K, K name "person"'),
+    ]
 
 
 class contact_point(RelationDefinition):
-    subject = 'Agent'
+    subject = 'OrganizationUnit'
     object = 'Agent'
     cardinality = '?*'
     inlined = True
-    constraints = [RQLVocabularyConstraint('O in_state ST, ST name "published"'),
-                   RQLVocabularyConstraint('NOT EXISTS(O agent_user UU)')]
-    description = _('set an agent of kind "person" as the contact point of an '
-                    'agent of kind "authority"')
+    constraints = [
+        RQLConstraint('S authority A, O authority A'),
+        RQLVocabularyConstraint('O in_state ST, ST name "published"'),
+    ]
+    description = _('set an agent as the contact point of an organization unit')
 
 
+class archival_unit(RelationDefinition):
+    subject = 'Organization'
+    object = 'OrganizationUnit'
+    cardinality = '?*'
+    description = _("the archival unit responsible for dealing with the organization's "
+                    "documents")
+    constraints = [RQLConstraint('O archival_role AR, AR name "archival"'),
+                   RQLVocabularyConstraint('O in_state ST, ST name "published"')]
+
+
+class archival_authority(ComputedRelation):
+    rule = 'S archival_unit OU, OU authority O'
+
+
+class ArchivalRole(EntityType):
+    """An archival role determines the kind of action (e.g. deposit or control)
+    an agent may perform on an archive entity.
+    """
+    __permissions__ = {
+        'read': ('managers', 'users', 'guests'),
+        'add': ('managers', ),
+        'update': ('managers', ),
+        'delete': ('managers', ),
+    }
+    name = String(required=True, unique=True, internationalizable=True)
+
+
+class archival_role(RelationDefinition):
+    subject = 'OrganizationUnit'
+    object = 'ArchivalRole'
+    cardinality = '**'
+    description = _("the organization unit's archival role (producer, control, etc.)")
+
+
+class use_email(RelationDefinition):
+    __permissions__ = {
+        'read': ('managers', 'users', 'guests',),
+        'add': ('managers', RRQLExpression('U has_update_permission S'),),
+        'delete': ('managers', RRQLExpression('U has_update_permission S'),),
+    }
+    subject = 'Agent'
+    object = 'EmailAddress'
+    cardinality = '*?'
+    composite = 'subject'
+    fulltext_container = 'subject'
+
+
+class phone_number(RelationDefinition):
+    __permissions__ = {
+        'read': ('managers', 'users', 'guests',),
+        'add': ('managers', RRQLExpression('U has_update_permission S'),),
+        'delete': ('managers', RRQLExpression('U has_update_permission S'),),
+    }
+    subject = 'Agent'
+    object = 'PhoneNumber'
+    cardinality = '*1'
+    composite = 'subject'
+
+
+# EAC-CPF model elements for AuthorityRecord #############################################
+
 class AgentFunction(EntityType):
-    """The function of an Agent"""
+    """The function of an AuthorityRecord"""
     name = String(fulltextindexed=True, internationalizable=True)
     description = RichString(fulltextindexed=True)
     __unique_together__ = [('name', 'function_agent')]
@@ -243,23 +414,15 @@ class AgentFunction(EntityType):
 
 class function_agent(RelationDefinition):
     subject = 'AgentFunction'
-    object = 'Agent'
+    object = 'AuthorityRecord'
     cardinality = '1*'
     composite = 'object'
     fulltext_container = 'object'
     inlined = True
 
 
-class phone_number(RelationDefinition):
-    subject = 'Agent'
-    object = 'PhoneNumber'
-    cardinality = '*1'
-    composite = 'subject'
-    fulltext_container = 'subject'
-
-
 class AgentPlace(EntityType):
-    """Qualified relation between an Agent and a PostalAddress"""
+    """Qualified relation between an AuthorityRecord and a PostalAddress"""
     name = String(fulltextindexed=True,
                   description=_('encoded information about the address (e.g. '
                                 '"Paris, France")'))
@@ -280,7 +443,7 @@ class place_address(RelationDefinition):
 
 class place_agent(RelationDefinition):
     subject = 'AgentPlace'
-    object = 'Agent'
+    object = 'AuthorityRecord'
     cardinality = '1*'
     inlined = True
     composite = 'object'
@@ -296,52 +459,26 @@ class AgentKind(EntityType):
     __permissions__ = {
         'read': ('managers', 'users', 'guests'),
         'add': ('managers', ),
-        'update': ('managers', ),
-        'delete': ('managers', ),
+        'update': (),
+        'delete': (),
     }
     name = String(required=True, unique=True, internationalizable=True)
 
 
 class agent_kind(RelationDefinition):
-    subject = 'Agent'
+    __permissions__ = {
+        'read': ('managers', 'users', 'guests'),
+        'add': ('managers', 'users'),
+        'delete': (RRQLExpression('O name "unknown-agent-kind"'),),
+    }
+    subject = 'AuthorityRecord'
     object = 'AgentKind'
     cardinality = '1*'
     inlined = True
 
 
-class ArchivalRole(EntityType):
-    """An archival role determines the kind of action (e.g. deposit or control)
-    an agent may perform on an archive entity.
-    """
-    __permissions__ = {
-        'read': ('managers', 'users', 'guests'),
-        'add': ('managers', ),
-        'update': ('managers', ),
-        'delete': ('managers', ),
-    }
-    name = String(required=True, unique=True, internationalizable=True)
-
-
-class archival_role(RelationDefinition):
-    subject = 'Agent'
-    object = 'ArchivalRole'
-    cardinality = '**'
-    description = _('the agent archival role (producer, control, etc.)')
-
-
-class archival_agent(RelationDefinition):
-    subject = 'Agent'
-    object = 'Agent'
-    cardinality = '?*'
-    description = _('the archival agent responsible for dealing with the agent '
-                    'documents')
-    constraints = [RQLConstraint('O archival_role AR, AR name "archival"'),
-                   RQLVocabularyConstraint('O in_state ST, ST name "published"'),
-                   RQLVocabularyConstraint('NOT EXISTS(O agent_user UU)')]
-
-
 class related_concept_scheme(RelationDefinition):
-    subject = 'Agent'
+    subject = 'OrganizationUnit'
     object = 'ConceptScheme'
     cardinality = '**'
     description = _('concept schemes used by the agent')
@@ -350,11 +487,26 @@ class related_concept_scheme(RelationDefinition):
 class _agent_relation(RelationDefinition):
     """Abstract relation between agents"""
     subject = None
-    object = ('Agent', 'ExternalUri')
+    object = ('AuthorityRecord', 'ExternalUri')
     cardinality = '1*'
     inlined = True
 
 
+class GeneralContext(EntityType):
+    """Information about the general social and cultural context of an agent"""
+    content = RichString(fulltextindexed=True)
+
+
+class general_context_of(RelationDefinition):
+    subject = 'GeneralContext'
+    object = 'AuthorityRecord'
+    cardinality = '1*'
+    inlined = True
+    composite = 'object'
+    fulltext_container = 'object'
+
+
+@xml_wrap
 @dated_entity_type
 class AssociationRelation(EntityType):
     """Association relation between agents"""
@@ -369,6 +521,7 @@ class association_to(_agent_relation):
     subject = 'AssociationRelation'
 
 
+@xml_wrap
 class ChronologicalRelation(EntityType):
     """Chronological relation between agents"""
     description = RichString()
@@ -382,6 +535,7 @@ class chronological_successor(_agent_relation):
     subject = 'ChronologicalRelation'
 
 
+@xml_wrap
 @dated_entity_type
 class HierarchicalRelation(EntityType):
     """Hierarchical relation between agents"""
@@ -398,27 +552,14 @@ class hierarchical_child(_agent_relation):
 
 class generated(RelationDefinition):
     subject = 'Activity'
-    object = ('Agent', 'Concept', 'ConceptScheme')
+    object = ('AuthorityRecord', 'Concept', 'ConceptScheme')
 
 
 class used(RelationDefinition):
     subject = 'Activity'
-    object = ('Agent', 'Concept', 'ConceptScheme')
+    object = ('AuthorityRecord', 'Concept', 'ConceptScheme')
 
 
-# ARK ##########################################################################
-
-class ark(RelationDefinition):
-    __permissions__ = RO_ATTR_PERMS
-    subject = ('Agent', 'ConceptScheme', 'Concept', 'SEDAProfile')
-    object = 'String'
-    description = _('ARK Identifier - will be generated if not specified')
-    constraints = [UniqueConstraint()]
-    cardinality = '11'
-
-
-# EAC-CPF model elements for Agent #############################################
-
 @dated_entity_type
 class Mandate(EntityType):
     """Reference text coming from an authority"""
@@ -451,7 +592,7 @@ class Occupation(EntityType):
 
 class occupation_agent(RelationDefinition):
     subject = 'Occupation'
-    object = 'Agent'
+    object = 'AuthorityRecord'
     cardinality = '1*'
     composite = 'object'
     fulltext_container = 'object'
@@ -461,42 +602,42 @@ class occupation_agent(RelationDefinition):
 
 class mandate_agent(RelationDefinition):
     subject = 'Mandate'
-    object = 'Agent'
+    object = 'AuthorityRecord'
     cardinality = '1*'
     composite = 'object'
     fulltext_container = 'object'
     inlined = True
-    description = _('mandate of an Agent')
+    description = _('mandate of an AuthorityRecord')
 
 
 class legal_status_agent(RelationDefinition):
     subject = 'LegalStatus'
-    object = 'Agent'
+    object = 'AuthorityRecord'
     cardinality = '1*'
     composite = 'object'
     fulltext_container = 'object'
     inlined = True
-    description = _('legal status of an Agent')
+    description = _('legal status of an AuthorityRecord')
 
 
 class structure_agent(RelationDefinition):
     subject = 'Structure'
-    object = 'Agent'
+    object = 'AuthorityRecord'
     cardinality = '1*'
     composite = 'object'
     fulltext_container = 'object'
     inlined = True
-    description = _('information about the structure of an Agent')
+    description = _('information about the structure of an AuthorityRecord')
 
 
 class history_agent(RelationDefinition):
     subject = 'History'
-    object = 'Agent'
+    object = 'AuthorityRecord'
     cardinality = '1*'
     composite = 'object'
     fulltext_container = 'object'
     inlined = True
-    description = _('information about the history of an Agent')
+    description = _('information about the history of an AuthorityRecord')
 
 
 class Citation(EntityType):
@@ -505,16 +646,18 @@ class Citation(EntityType):
 
 
 class has_citation(RelationDefinition):
-    subject = ('Mandate', 'Occupation')
+    subject = ('GeneralContext', 'Mandate', 'Occupation', 'History',
+               'AgentFunction', 'LegalStatus', 'AgentPlace')
     object = 'Citation'
     cardinality = '*?'
     composite = 'subject'
     description = _('reference to an external citation resource')
 
 
+@xml_wrap
 @dated_entity_type
 class EACResourceRelation(EntityType):
-    """Represent a relation between an Agent and a remote resource in the
+    """Represent a relation between an AuthorityRecord and a remote resource in the
     EAC-CPF model.
     """
     agent_role = String(description=_('type of relation the agent has to the resource'),
@@ -526,7 +669,7 @@ class EACResourceRelation(EntityType):
 
 class resource_relation_agent(RelationDefinition):
     subject = 'EACResourceRelation'
-    object = 'Agent'
+    object = 'AuthorityRecord'
     cardinality = '1*'
     inlined = True
     composite = 'object'
@@ -540,8 +683,9 @@ class resource_relation_resource(RelationDefinition):
     inlined = True
 
 
+@xml_wrap
 class EACSource(EntityType):
-    """A source used to establish the description of an Agent"""
+    """A source used to establish the description of an AuthorityRecord"""
     title = String(fulltextindexed=True)
     url = String()
     description = RichString(fulltextindexed=True)
@@ -549,7 +693,7 @@ class EACSource(EntityType):
 
 class source_agent(RelationDefinition):
     subject = 'EACSource'
-    object = 'Agent'
+    object = 'AuthorityRecord'
     cardinality = '1*'
     composite = 'object'
     fulltext_container = 'object'
@@ -574,6 +718,73 @@ class equivalent_concept(RelationDefinition):
     inlined = False
 
 
+# ARK ##########################################################################
+
+class ark(RelationDefinition):
+    __permissions__ = RO_ATTR_PERMS
+    subject = (
+        'Agent',
+        'AuthorityRecord',
+        'Concept',
+        'ConceptScheme',
+        'OrganizationUnit',
+        'SEDAProfile',
+    )
+    object = 'String'
+    description = _('ARK Identifier - will be generated if not specified')
+    constraints = [UniqueConstraint()]
+    cardinality = '11'
+
+
+class ArkNameAssigningAuthority(EntityType):
+    """Name Assigning Authority (NAA) for ARK generation."""
+    who = String(required=True, unique=True,
+                 description=_('official organization name'))
+    what = Int(required=True, unique=True,
+               description=_('Name Assigning Authority Number (NAAN)'))
+
+
+class _ark_naa(RelationDefinition):
+    name = 'ark_naa'
+    object = 'ArkNameAssigningAuthority'
+    inlined = True
+
+
+class organization_ark_naa(_ark_naa):
+    __permissions__ = {
+        'read': ('managers', 'users', 'guests'),
+        'add': ('managers', ),
+        'delete': ('managers', ),
+    }
+    subject = 'Organization'
+    cardinality = '?*'
+    description = _("ARK identifier Name Assigning Authority (NAA) - "
+                    "you'll need one to start creating objects")
+
+
+class mandatory_ark_naa(_ark_naa):
+    __permissions__ = {
+        'read': ('managers', 'users', 'guests'),
+        'add': ('managers', 'users',),
+        'delete': (),
+    }
+    subject = ('AuthorityRecord', 'SEDAProfile')
+    cardinality = '1*'
+    description = _("ARK identifier Name Assigning Authority (NAA)")
+
+
+class optional_ark_naa(_ark_naa):
+    __permissions__ = {
+        'read': ('managers', 'users', 'guests'),
+        'add': ('managers', 'users',),
+        'delete': (),
+    }
+    subject = ('ConceptScheme', 'SKOSSource')
+    cardinality = '?*'
+    description = _("ARK identifier Name Assigning Authority (NAA), "
+                    "necessary to create ARK for concepts which don't have one yet.")
+
+
 # SEDA #######################################################################
 
 @publication_permissions
@@ -590,9 +801,9 @@ class SEDAProfile(WorkflowableEntityType):
                                                    'update': ()})
 
 
-class seda_transferring_agent(RelationDefinition):  # XXX Agent use_profile Profile
+class seda_transferring_agent(RelationDefinition):  # XXX OrganizationUnit use_profile Profile
     subject = 'SEDAProfile'
-    object = 'Agent'
+    object = 'OrganizationUnit'
     cardinality = '**'
     constraints = [RQLConstraint('O archival_role R, R name "deposit"')]
 
@@ -732,7 +943,6 @@ class SEDAAppraisalRuleDuration(EntityType):
 
     http://www.archivesdefrance.culture.gouv.fr/seda/api/seda_v1-0_archive_xsd_Complex_Type_AppraisalRuleType.htm#AppraisalRuleType_Duration
     """
-    value = String(description=_('ISO-8601 syntax, for exemple P10Y for 10 years'))
 
 
 @seda_profile_element()
@@ -759,6 +969,15 @@ class seda_appraisal_rule_duration(RelationDefinition):
     inlined = True
 
 
+class seda_appraisal_rule_duration_value(RelationDefinition):
+    subject = 'SEDAAppraisalRuleDuration'
+    object = 'Concept'
+    cardinality = '?*'
+    inlined = True
+    constraints = [RQLConstraint('O in_scheme CS, CS scheme_relation CR, CR name '
+                                 '"seda_appraisal_rule_duration", CR is CWRType')]
+
+
 class seda_appraisal_rule_code(RelationDefinition):
     subject = 'SEDAAppraisalRule'
     object = 'SEDAAppraisalRuleCode'
