@@ -1,4 +1,4 @@
-# copyright 2015 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# copyright 2015-2016 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 # contact http://www.logilab.fr -- mailto:contact@logilab.fr
 #
 # This program is free software: you can redistribute it and/or modify it under
@@ -16,7 +16,7 @@
 """cubicweb-saem-ref test for OAI-PMH export"""
 
 from collections import namedtuple
-from datetime import date, datetime, timedelta
+from datetime import datetime, timedelta
 from functools import wraps
 import time
 
@@ -28,9 +28,9 @@ from logilab.common import tempattr
 
 from cubicweb.devtools.testlib import CubicWebTC
 
-from cubes.saem_ref import isodate
-from cubes.saem_ref.entities.oai import OAIComponent
-from cubes.saem_ref.views.oai import OAIError, OAIRequest
+from cubes.oaipmh import isodate
+from cubes.oaipmh.entities import OAIComponent
+from cubes.oaipmh.views import OAIError, OAIRequest
 
 import testutils
 
@@ -64,9 +64,9 @@ class OAIComponentTC(CubicWebTC, OAITestMixin):
             oai = self.oai_component(cnx)
             setspecs = oai.__setspecs__
             self.assertCountEqual(setspecs.keys(),
-                                  ['agent', 'conceptscheme', 'profile', 'concept'])
-            self.assertCountEqual(setspecs['agent'].keys(),
-                                  ['kind', 'role'])
+                                  ['agent', 'organizationunit',
+                                   'conceptscheme', 'profile', 'concept'])
+            self.assertCountEqual(setspecs['organizationunit'].keys(), ['role'])
             self.assertCountEqual(setspecs['profile'].keys(),
                                   ['transferring_agent'])
             self.assertEqual(setspecs['conceptscheme'].keys(), [])
@@ -80,18 +80,22 @@ class OAIComponentTC(CubicWebTC, OAITestMixin):
 
     def test_setspecs(self):
         with self.admin_access.repo_cnx() as cnx:
-            arch = testutils.agent(cnx, u'arch', archival_roles=[u'archival'])
-            ark = testutils.agent(cnx, u'dep', archival_roles=[u'deposit'],
-                                  archival_agent=arch).ark
+            testutils.organization_unit(
+                cnx, u'arch', archival_roles=[u'archival'])
+            ark = testutils.organization_unit(
+                cnx, u'dep', archival_roles=[u'deposit']).ark
+            scheme = testutils.setup_scheme(cnx, u'test', u'lab')
             cnx.commit()
-            scheme = setup_scheme(cnx, u'test', u'lab')
             publish_entity(scheme)
             expected = (
                 'agent',
-                'agent:kind:person', 'agent:kind:authority', 'agent:kind:family',
-                'agent:role:producer', 'agent:role:deposit',
-                'agent:role:archival', 'agent:role:control',
-                'agent:role:enquirer', 'agent:role:seda-actor',
+                'organizationunit',
+                'organizationunit:role:producer',
+                'organizationunit:role:deposit',
+                'organizationunit:role:archival',
+                'organizationunit:role:control',
+                'organizationunit:role:enquirer',
+                'organizationunit:role:seda-actor',
                 'conceptscheme',
                 'profile', 'profile:transferring_agent:%s' % ark,
                 'concept', 'concept:in_scheme:%s' % scheme.ark
@@ -102,36 +106,25 @@ class OAIComponentTC(CubicWebTC, OAITestMixin):
 
 
 def setup_agents(cnx):
-    """Create three agents (alice, bob and draft), publish alice and bob but not draft, and return
-    the (ark, url, eid) attributes of alice and bob.
+    """Create three agents (alice, bobby and draft), publish alice and bobby but not draft, and return
+    the (ark, url, eid) attributes of alice and bobby.
     """
-    alice = testutils.agent(cnx, u'alice', archival_roles=[u'producer'])
+    alice = testutils.organization_unit(cnx, u'alice',
+                                        archival_roles=[u'producer'])
     cnx.commit()
     publish_entity(alice)
-    bob = testutils.agent(cnx, u'bob', archival_roles=[u'enquirer'],
-                          kind=u'family')
+    bobby = testutils.organization_unit(cnx, u'bobby',
+                                        archival_roles=[u'enquirer'])
     cnx.commit()
-    publish_entity(bob)
-    testutils.agent(cnx, u'draft', kind=u'family',
-                    archival_roles=[u'enquirer'])
+    publish_entity(bobby)
+    testutils.organization_unit(cnx, u'draft', archival_roles=[u'enquirer'])
     cnx.commit()
     return (
         EntityInfo(alice.ark, alice.absolute_url(), alice.eid),
-        EntityInfo(bob.ark, bob.absolute_url(), bob.eid),
+        EntityInfo(bobby.ark, bobby.absolute_url(), bobby.eid),
     )
 
 
-def setup_scheme(cnx, title, *labels):
-    """Return info new concept scheme"""
-    scheme = cnx.create_entity('ConceptScheme', title=title)
-    for label in labels:
-        cnx.create_entity(
-            'Label', label=label,
-            label_of=cnx.create_entity('Concept', in_scheme=scheme))
-    cnx.commit()
-    return scheme
-
-
 class OAIRequestTC(CubicWebTC):
 
     def test_rset_from_setspec(self):
@@ -140,32 +133,32 @@ class OAIRequestTC(CubicWebTC):
             return setup_agents(cnx)
         with self.admin_access.web_request() as req:
             for setspec, msg in [
-                ('agent:zen', 'invalid set specifier agent:zen'),
-                ('agent:role:control', (
+                ('organizationunit:zen', 'invalid set specifier organizationunit:zen'),
+                ('organizationunit:role:control', (
                     'The combination of the values of the from, until, and '
                     'set arguments results in an empty list.')),
                 ('conceptscheme:haha', 'invalid set specifier conceptscheme:haha'),
                 ('scregneugneu', 'invalid set specifier scregneugneu'),
             ]:
-                self.set_description(setspec)
-                oairq = OAIRequest('http://testing.fr', setspec=setspec)
-                with self.assertRaises(OAIError) as cm:
-                    oairq.rset_from_setspec(req)
-                self.assertEqual(cm.exception.errors, {'noRecordsMatch': msg})
+                with self.subTest(setspec=setspec):
+                    oairq = OAIRequest('http://testing.fr', setspec=setspec)
+                    with self.assertRaises(OAIError) as cm:
+                        oairq.rset_from_setspec(req)
+                    self.assertEqual(cm.exception.errors, {'noRecordsMatch': msg})
 
     def test_rset_from_setspec_token(self):
         with self.admin_access.repo_cnx() as cnx:
-            alice, bob = setup_agents(cnx)
+            alice, bobby = setup_agents(cnx)
 
         def oairq(**kwargs):
-            return OAIRequest('http://testing.fr', setspec='agent', **kwargs)
+            return OAIRequest('http://testing.fr', setspec='organizationunit', **kwargs)
 
         with self.admin_access.web_request() as req:
             with tempattr(OAIComponent, 'max_result_size', 1):
                 rset, token = oairq().rset_from_setspec(req)
                 self.assertEqual(len(rset), 1)
-                self.assertEqual(token, str(bob.eid))
-                rset, token = oairq(resumption_token=str(bob.eid)).rset_from_setspec(req)
+                self.assertEqual(token, str(bobby.eid))
+                rset, token = oairq(resumption_token=str(bobby.eid)).rset_from_setspec(req)
                 self.assertEqual(len(rset), 1)
                 self.assertIsNone(token)
             # All results are returned, the resumptionToken is None.
@@ -236,23 +229,24 @@ class OAIPMHViewsTC(CubicWebTC, OAITestMixin, testutils.XmlTestMixin):
                       result)
 
     def _setup_agents_deleted(self):
-        alice, bob = self._setup_agents()
+        alice, bobby = self._setup_agents()
         with self.admin_access.repo_cnx() as cnx:
-            entity = cnx.entity_from_eid(bob.eid)
+            entity = cnx.entity_from_eid(bobby.eid)
             # ensure transition date is greater by at least 1s
             time.sleep(1)
             date_before = entity.modification_date.replace(tzinfo=pytz.utc)
             deprecate_entity(entity)
-        return alice, bob, date_before
+        return alice, bobby, date_before
 
     def _setup_schemes(self):
         with self.admin_access.repo_cnx() as cnx:
-            pscheme = setup_scheme(cnx, u'public',
-                                   u'public label', u'other public label')
-            publish_entity(pscheme)
+            pscheme = testutils.setup_scheme(cnx, u'public',
+                                             u'public label', u'other public label')
             # draft ConceptScheme
-            dscheme = setup_scheme(cnx, u'draft',
-                                   u'draft label', u'other draft label')
+            dscheme = testutils.setup_scheme(cnx, u'draft',
+                                             u'draft label', u'other draft label')
+            cnx.commit()
+            publish_entity(pscheme)
             return (
                 EntityInfo(pscheme.ark, pscheme.absolute_url(), pscheme.eid),
                 EntityInfo(dscheme.ark, dscheme.absolute_url(), dscheme.eid),
@@ -282,11 +276,12 @@ class OAIPMHViewsTC(CubicWebTC, OAITestMixin, testutils.XmlTestMixin):
             result = self.oai_request(req, verb='ListSets')
         self.assertIn('<request verb="ListSets">http://testing.fr/cubicweb/oai</request>',
                       result)
-        for spec in ('agent', 'conceptscheme', 'profile', 'concept'):
+        for spec in ('agent', 'organizationunit', 'conceptscheme', 'profile',
+                     'concept'):
             self.assertIn('<setSpec>{0}</setSpec>'.format(spec),
                           result)
-        self.assertIn(('<setSpec>agent:role:deposit</setSpec>'
-                       '<setName>An agent with deposit archival role</setName>'),
+        self.assertIn(('<setSpec>organizationunit:role:deposit</setSpec>'
+                       '<setName>An organization unit with deposit archival role</setName>'),
                       result)
         self.assertIn('<setSpec>concept:in_scheme:{0}</setSpec>'.format(pscheme.ark),
                       result)
@@ -302,85 +297,87 @@ class OAIPMHViewsTC(CubicWebTC, OAITestMixin, testutils.XmlTestMixin):
                        '</error>'),
                       result)
 
-    def _setup_profile(self):
+    def _setup_profile(self, **kwargs):
         """Create a "published" SEDAProfile and return its identifier"""
         with self.admin_access.repo_cnx() as cnx:
-            profile = testutils.setup_seda_profile(cnx)
+            profile = testutils.setup_seda_profile(cnx, **kwargs)
             cnx.commit()
             publish_entity(profile)
             return profile.ark
 
     def test_resumptiontoken_listidentifiers(self):
-        alice, bob = self._setup_agents()
+        alice, bobby = self._setup_agents()
 
         def check_request(expected_arks, token=None):
             with self.admin_access.web_request() as req:
                 result = self.oai_request(
-                    req, verb='ListIdentifiers', set='agent',
+                    req, verb='ListIdentifiers', set='organizationunit',
                     resumptionToken=token)
                 # Ensure there are as many <identifier> tag than expected items.
                 self.assertEqual(result.count('<identifier>'), len(expected_arks))
                 for ark in expected_arks:
                     self.assertIn(
-                        '<identifier>{0}</identifier>'.format(ark), result)
+                        '<identifier>ark:/{0}</identifier>'.format(ark), result)
                 return result
 
-        self.set_description('less results than pagination limit')
-        result = check_request([alice.ark, bob.ark])
+        # less results than pagination limit
+        result = check_request([alice.ark, bobby.ark])
         # All results returned, no resumptionToken is request -> no
         # resumptionToken in response.
         self.assertNotIn('resumptionToken', result)
         with tempattr(OAIComponent, 'max_result_size', 1):
-            self.set_description('max_result_size = 1, 1st batch')
+            # max_result_size = 1, 1st batch
             result = check_request([alice.ark])
-            self.set_description('token + expirationDate')
+            # token + expirationDate
             root = etree.fromstring(result)
             token = root.find('.//{%s}resumptionToken' % root.nsmap[None])
             date = parse_date(token.attrib['expirationDate'])
             self.assertLess(datetime.now(pytz.utc) + timedelta(hours=1) - date,
                             timedelta(seconds=1))
-            self.assertEqual(token.text, str(bob.eid))
-            self.set_description('max_result_size = 1, 2nd batch')
-            result = check_request([bob.ark], token=str(bob.eid))
-            self.set_description('max_result_size = 1, no more result')
+            self.assertEqual(token.text, str(bobby.eid))
+            # max_result_size = 1, 2nd batch
+            result = check_request([bobby.ark], token=str(bobby.eid))
+            # max_result_size = 1, no more result
             # No more results -> empty resumptionToken.
             self.assertIn('<resumptionToken/>', result)
             # Empty result, probably a badResumptionToken.
-            result = check_request([], token=str(bob.eid + 1))
+            result = check_request([], token=str(bobby.eid + 1))
             self.assertIn('badResumptionToken', result)
 
     def test_listidentifiers(self):
-        alice, bob = [x.ark for x in self._setup_agents()]
+        alice, bobby = [x.ark for x in self._setup_agents()]
         profile_id = self._setup_profile()
         with self.admin_access.web_request() as req:
-            result = self.oai_request(req, verb='ListIdentifiers', set='agent')
-            self.assertIn('<identifier>{0}</identifier>'.format(bob), result)
-            self.assertIn('<identifier>{0}</identifier>'.format(alice), result)
+            result = self.oai_request(req, verb='ListIdentifiers',
+                                      set='organizationunit')
+            self.assertIn('<identifier>ark:/{0}</identifier>'.format(bobby), result)
+            self.assertIn('<identifier>ark:/{0}</identifier>'.format(alice), result)
             draft = req.execute('Any A WHERE X ark A, X name "draft"')[0][0]
             self.assertNotIn(draft, result)
             result = self.oai_request(req, verb='ListIdentifiers',
-                                      set='agent:role:producer')
-            self.assertIn('<identifier>{0}</identifier>'.format(alice),
+                                      set='organizationunit:role:producer')
+            self.assertIn('<identifier>ark:/{0}</identifier>'.format(alice),
                           result)
-            self.assertNotIn('<identifier>{0}</identifier>'.format(bob),
+            self.assertNotIn('<identifier>ark:/{0}</identifier>'.format(bobby),
                              result)
-            result = self.oai_request(req, verb='ListIdentifiers', set='agent:kind:person')
-            self.assertIn('<identifier>{0}</identifier>'.format(alice), result)
-            self.assertNotIn('<identifier>{0}</identifier>'.format(bob), result)
+            result = self.oai_request(req, verb='ListIdentifiers',
+                                      set='organizationunit:role:producer')
+            self.assertIn('<identifier>ark:/{0}</identifier>'.format(alice), result)
+            self.assertNotIn('<identifier>ark:/{0}</identifier>'.format(bobby), result)
             result = self.oai_request(req, verb='ListIdentifiers', set='conceptscheme')
             result = self.oai_request(req, verb='ListIdentifiers', set='profile')
-            self.assertIn('<identifier>{0}</identifier>'.format(profile_id), result)
+            self.assertIn('<identifier>ark:/{0}</identifier>'.format(profile_id), result)
 
     def test_listidentifiers_setspec_wfstate(self):
         """Check workflow state condition is statisfied with a multi-level
-        setspec `agent:role:enquirer`.
+        setspec `organizationunit:role:enquirer`.
         """
-        alice, bob = [x.ark for x in self._setup_agents()]
-        profile_id = self._setup_profile()
+        alice, bobby = [x.ark for x in self._setup_agents()]
+        self._setup_profile()
         with self.admin_access.web_request() as req:
             result = self.oai_request(req, verb='ListIdentifiers',
-                                      set='agent:role:enquirer')
-            self.assertIn('<identifier>{0}</identifier>'.format(bob), result)
+                                      set='organizationunit:role:enquirer')
+            self.assertIn('<identifier>ark:/{0}</identifier>'.format(bobby), result)
             draft = req.execute('Any A WHERE X ark A, X name "draft"')[0][0]
             self.assertNotIn(draft, result)
 
@@ -388,11 +385,12 @@ class OAIPMHViewsTC(CubicWebTC, OAITestMixin, testutils.XmlTestMixin):
         """Check <header> element (status attribute, datestamp) for deleted
         records in ListIdentifiers response.
         """
-        alice, bob, date_before = self._setup_agents_deleted()
+        alice, bobby, date_before = self._setup_agents_deleted()
         with self.admin_access.web_request() as req:
-            result = self.oai_request(req, verb='ListIdentifiers', set='agent')
+            result = self.oai_request(req, verb='ListIdentifiers',
+                                      set='organizationunit')
             self.assertIn(
-                '<identifier>{0}</identifier>'.format(bob.ark), result)
+                '<identifier>ark:/{0}</identifier>'.format(bobby.ark), result)
             self.assertIn('<header status="deleted">', result)
             root = etree.fromstring(result)
             ns = root.nsmap[None]
@@ -414,38 +412,35 @@ class OAIPMHViewsTC(CubicWebTC, OAITestMixin, testutils.XmlTestMixin):
 
     @no_validate_xml
     def test_listrecords(self):
-        alice, bob = self._setup_agents()
+        alice, bobby = self._setup_agents()
         with self.admin_access.web_request() as req:
-            result = self.oai_request(req, verb='ListRecords', set='agent')
+            result = self.oai_request(req, verb='ListRecords',
+                                      set='organizationunit')
             self.assertIn(
-                '<rdf:Description rdf:about="{0}">'.format(bob.url), result)
+                '<rdf:Description rdf:about="{0}">'.format(bobby.url), result)
             self.assertIn(
                 '<rdf:Description rdf:about="{0}">'.format(alice.url), result)
             draft = req.execute('Any A WHERE X ark A, X name "draft"')[0][0]
             self.assertNotIn(draft, result)
             result = self.oai_request(
-                req, verb='ListRecords', set='agent:role:producer')
+                req, verb='ListRecords', set='organizationunit:role:producer')
             self.assertIn(
                 '<rdf:Description rdf:about="{0}">'.format(alice.url), result)
-            self.assertNotIn(bob.url, result)
-            result = self.oai_request(req, verb='ListRecords', set='agent:kind:person')
-            self.assertIn(
-                '<rdf:Description rdf:about="{0}">'.format(alice.url), result)
-            self.assertNotIn(bob.url, result)
+            self.assertNotIn(bobby.url, result)
 
     @no_validate_xml
     def test_listrecord_deleted(self):
         """Check absence of <metadata> or <about> elements for deleted
         records in ListRecords response.
         """
-        alice, bob, _ = self._setup_agents_deleted()
+        alice, bobby, _ = self._setup_agents_deleted()
         with self.admin_access.web_request() as req:
-            result = self.oai_request(req, verb='ListRecords', set='agent')
-            # no metadata for bob
+            result = self.oai_request(req, verb='ListRecords', set='organizationunit')
+            # no metadata for bobby
             self.assertNotIn(
-                '<rdf:Description rdf:about="{0}">'.format(bob.url), result)
+                '<rdf:Description rdf:about="{0}">'.format(bobby.url), result)
             # but record present
-            self.assertIn('<identifier>{0}</identifier>'.format(bob.ark), result)
+            self.assertIn('<identifier>ark:/{0}</identifier>'.format(bobby.ark), result)
             self.assertIn(
                 '<rdf:Description rdf:about="{0}">'.format(alice.url), result)
 
@@ -462,7 +457,11 @@ class OAIPMHViewsTC(CubicWebTC, OAITestMixin, testutils.XmlTestMixin):
     def test_listidentifiers_profile_setspec(self):
         """Check setspec restriction for SEDAProfile."""
         profile_id = self._setup_profile()
-        profile2_id = self._setup_profile()
+        with self.admin_access.cnx() as cnx:
+            tagent_eid = testutils.organization_unit(
+                cnx, u'blah', archival_roles=[u'deposit']).eid
+            cnx.commit()
+        profile2_id = self._setup_profile(seda_transferring_agent=tagent_eid)
         with self.admin_access.web_request() as req:
             transferring_agent_ark = req.execute(
                 'Any K WHERE P ark "%s", P seda_transferring_agent A, A ark K'
@@ -470,13 +469,13 @@ class OAIPMHViewsTC(CubicWebTC, OAITestMixin, testutils.XmlTestMixin):
             result = self.oai_request(
                 req, verb='ListIdentifiers',
                 set='profile:transferring_agent:' + transferring_agent_ark)
-            self.assertIn('<identifier>{0}</identifier>'.format(profile_id), result)
+            self.assertIn('<identifier>ark:/{0}</identifier>'.format(profile_id), result)
             self.assertNotIn(profile2_id, result)
         # Check that without a setspec, we get all profiles.
         with self.admin_access.web_request() as req:
             result = self.oai_request(req, verb='ListIdentifiers', set='profile')
-            self.assertIn('<identifier>{0}</identifier>'.format(profile_id), result)
-            self.assertIn('<identifier>{0}</identifier>'.format(profile2_id), result)
+            self.assertIn('<identifier>ark:/{0}</identifier>'.format(profile_id), result)
+            self.assertIn('<identifier>ark:/{0}</identifier>'.format(profile2_id), result)
 
     @no_validate_xml
     def test_getrecord_profile(self):
@@ -486,7 +485,7 @@ class OAIPMHViewsTC(CubicWebTC, OAITestMixin, testutils.XmlTestMixin):
             self.assertIn('<error code="badArgument">', result)
             result = self.oai_request(req, verb='GetRecord', identifier='999')
             self.assertIn('<error code="idDoesNotExist"', result)
-            result = self.oai_request(req, verb='GetRecord', identifier=profile_id)
+            result = self.oai_request(req, verb='GetRecord', identifier='ark:/' + profile_id)
             self.assertIn('<metadata><xsd:schema', result)
             self.assertIn('fr:gouv:ae:archive:draft:standard_echange_v0.2', result)
 
@@ -499,20 +498,34 @@ class OAIPMHViewsTC(CubicWebTC, OAITestMixin, testutils.XmlTestMixin):
             publish_entity(agent)
             ark = agent.ark
         with self.admin_access.web_request() as req:
-            result = self.oai_request(req, verb='GetRecord', identifier=ark)
+            result = self.oai_request(req, verb='GetRecord', identifier='ark:/' + ark)
             self.assertIn('name>toto</', result)
             self.assertIn('<rdf:type rdf:resource="http://xmlns.com/foaf/0.1/Person"/>',
                           result)
 
+    @no_validate_xml
+    def test_getrecord_organizationunit(self):
+        self._debug_xml = True
+        with self.admin_access.repo_cnx() as cnx:
+            orgu = testutils.organization_unit(cnx, u'toto')
+            cnx.commit()
+            publish_entity(orgu)
+            ark = orgu.ark
+        with self.admin_access.web_request() as req:
+            result = self.oai_request(req, verb='GetRecord', identifier='ark:/' + ark)
+            self.assertIn('name>toto</', result)
+            self.assertIn('<rdf:type rdf:resource="http://www.w3.org/ns/org#OrganizationUnit"/>',
+                          result)
+
     def test_getrecord_deleted(self):
         """Check absence of <metadata> or <about> elements for deleted
         records in GetRecord response.
         """
-        alice, bob, _ = self._setup_agents_deleted()
+        alice, bobby, _ = self._setup_agents_deleted()
         with self.admin_access.web_request() as req:
-            result = self.oai_request(req, verb='GetRecord', identifier=bob.ark)
+            result = self.oai_request(req, verb='GetRecord', identifier='ark:/' + bobby.ark)
             # record is present
-            self.assertIn('<identifier>{0}</identifier>'.format(bob.ark), result)
+            self.assertIn('<identifier>ark:/{0}</identifier>'.format(bobby.ark), result)
             # but not data
             self.assertNotIn('metadata', result)
             self.assertNotIn('rdf', result)
@@ -521,97 +534,10 @@ class OAIPMHViewsTC(CubicWebTC, OAITestMixin, testutils.XmlTestMixin):
         self._setup_agents()
         with self.admin_access.web_request() as req:
             draft = req.execute('Any A WHERE X ark A, X name "draft"')[0][0]
-            result = self.oai_request(req, verb='GetRecord', identifier=draft)
+            result = self.oai_request(req, verb='GetRecord', identifier='ark:/' + draft)
             self.assertIn('idDoesNotExist', result)
 
-    def test_from_until_error(self):
-        with self.admin_access.web_request() as req:
-            dates = {'from': date(2013, 1, 2).isoformat(),
-                     'until': date(2011, 1, 2).isoformat()}
-            result = self.oai_request(req, verb='ListIdentifiers', set='agent',
-                                      **dates)
-            self.assertIn('<error code="badArgument">the from argument', result)
-
-    def test_from_until_listidentifiers(self):
-        alice, bob = self._setup_agents()
-        with self.admin_access.repo_cnx() as cnx:
-            alice_date = cnx.entity_from_eid(alice.eid).modification_date
-            bob_date = alice_date + timedelta(seconds=2)
-            cnx.entity_from_eid(bob.eid).cw_set(modification_date=bob_date)
-            cnx.commit()
-        in_between_date = isodate(alice_date + timedelta(seconds=1))
-        self.set_description('from only')
-        with self.admin_access.web_request() as req:
-            result = self.oai_request(req, verb='ListIdentifiers', set='agent',
-                                      **{'from': in_between_date})
-            self.assertIn(bob.ark, result)
-            self.assertNotIn(alice.ark, result)
-        self.set_description('until only')
-        with self.admin_access.web_request() as req:
-            result = self.oai_request(req, verb='ListIdentifiers', set='agent',
-                                      **{'until': in_between_date})
-            self.assertNotIn(bob.ark, result)
-            self.assertIn(alice.ark, result)
-        self.set_description('from and until')
-        with self.admin_access.web_request() as req:
-            result = self.oai_request(req, verb='ListIdentifiers', set='agent',
-                                      **{'from': '1970-12-1',
-                                         'until': '2099-01-02'})
-            self.assertIn(bob.ark, result)
-            self.assertIn(alice.ark, result)
-        self.set_description('no record match')
-        with self.admin_access.web_request() as req:
-            result = self.oai_request(req, verb='ListIdentifiers', set='agent',
-                                      **{'until': '1970-12-31'})
-            self.assertIn('<error code="noRecordsMatch">The combi', result)
-
-    def test_from_until_listidentifiers_setspec(self):
-        """Check from/until restriction with a multi-level setspec
-        `agent:kind:family`
-        """
-        alice, bob = self._setup_agents()
-        with self.admin_access.repo_cnx() as cnx:
-            bob_date = cnx.entity_from_eid(bob.eid).modification_date
-            marcel = cnx.find('Agent', name=u'draft').one()
-            publish_entity(marcel)
-            marcel_date = bob_date + timedelta(seconds=2)
-            marcel.cw_set(name=u'marcel', modification_date=marcel_date)
-            marcel_ark = marcel.ark
-            cnx.commit()
-        in_between_date = isodate(bob_date + timedelta(seconds=1))
-        self.set_description('from only')
-        with self.admin_access.web_request() as req:
-            result = self.oai_request(req, verb='ListIdentifiers',
-                                      set='agent:kind:family',
-                                      **{'from': in_between_date})
-            self.assertNotIn(bob.ark, result)
-            self.assertIn(marcel_ark, result)
-            self.assertNotIn(alice.ark, result)
-        self.set_description('until only')
-        with self.admin_access.web_request() as req:
-            result = self.oai_request(req, verb='ListIdentifiers',
-                                      set='agent:kind:family',
-                                      **{'until': in_between_date})
-            self.assertIn(bob.ark, result)
-            self.assertNotIn(marcel_ark, result)
-            self.assertNotIn(alice.ark, result)
-        self.set_description('from and until')
-        with self.admin_access.web_request() as req:
-            result = self.oai_request(req, verb='ListIdentifiers',
-                                      set='agent:kind:family',
-                                      **{'from': '1970-12-1',
-                                         'until': '2099-01-02'})
-            self.assertIn(bob.ark, result)
-            self.assertIn(marcel_ark, result)
-            self.assertNotIn(alice.ark, result)
-        self.set_description('no record match')
-        with self.admin_access.web_request() as req:
-            result = self.oai_request(req, verb='ListIdentifiers',
-                                      set='agent:kind:family',
-                                      **{'until': '1970-12-31'})
-            self.assertIn('<error code="noRecordsMatch">The combi', result)
-
 
 if __name__ == '__main__':
-    from logilab.common.testlib import unittest_main
-    unittest_main()
+    import unittest
+    unittest.main()
