@@ -18,17 +18,30 @@ from cubicweb.devtools.testlib import CubicWebTC
 
 
 class ArkGeneratorTC(CubicWebTC):
+
     def test_eid(self):
         with self.admin_access.repo_cnx() as cnx:
-            generator = self.vreg['adapters'].select('IARKGenerator', cnx, eid=12)
-            self.assertEqual(generator.generate_ark(), 'saemref-test/000000012')
+            for etype, expected in (
+                ("AuthorityRecord", u"r000000042"),
+                ("SEDAProfile", u"p000000042"),
+                ("Concept", u"c000000042"),
+                ("ConceptScheme", u"v000000042"),
+                ("Organization", u"o000000042"),
+                ("OrganizationUnit", u"ou000000042"),
+                ("Agent", u"oa000000042"),
+                ("CWUser", u"000000042"),
+            ):
+                generator = self.vreg['adapters'].select('IARKGenerator', cnx, eid=42, etype=etype,
+                                                         naa_what='saemref-test')
+                self.assertEqual(generator.generate_ark(), 'saemref-test/{0}'.format(expected))
 
     def test_ext_identifier(self):
         with self.admin_access.repo_cnx() as cnx:
-            generator = self.vreg['adapters'].select('IARKGenerator', cnx)
-            self.assertEqual(generator.generate_ark(), 'saemref-test/ext-000000001')
+            generator = self.vreg['adapters'].select('IARKGenerator', cnx,
+                                                     naa_what='saemref-test')
+            self.assertEqual(generator.generate_ark(), 'saemref-test/a000000001')
 
 
 if __name__ == '__main__':
-    from logilab.common.testlib import unittest_main
-    unittest_main()
+    import unittest
+    unittest.main()
