@@ -1,13 +1,26 @@
+# copyright 2015-2016 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# contact http://www.logilab.fr -- mailto:contact@logilab.fr
+#
+# This program is free software: you can redistribute it and/or modify it under
+# the terms of the GNU Lesser General Public License as published by the Free
+# Software Foundation, either version 2.1 of the License, or (at your option)
+# any later version.
+#
+# This program is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
+# details.
+#
+# You should have received a copy of the GNU Lesser General Public License along
+# with this program. If not, see <http://www.gnu.org/licenses/>.
 """cubicweb-saem_ref unit tests for hooks"""
 
 from datetime import datetime, timedelta
 from time import mktime
+import unittest
 
 import pytz
 
-from logilab.common.testlib import TestCase
-from yams import ValidationError
-
 from cubicweb.devtools.testlib import CubicWebTC
 
 from cubes.saem_ref.hooks import extract_ark
@@ -17,17 +30,12 @@ import testutils
 YESTERDAY = datetime.now(tz=pytz.utc) - timedelta(days=1)
 
 
-def create_agent(cnx, name, kind='person', **kwargs):
-    akind = cnx.find('AgentKind', name=kind).one()
-    return cnx.create_entity('Agent', name=name, agent_kind=akind, **kwargs)
-
-
 class SAEMRefHooksTC(CubicWebTC):
 
     def assertMDNow(self, entity):
         entity.cw_clear_all_caches()
         self.assertAlmostEqual(mktime(entity.modification_date.timetuple()),
-                               mktime(datetime.utcnow().timetuple()), delta=60)
+                               mktime(datetime.now(tz=pytz.utc).timetuple()), delta=60)
 
     def resetMD(self, cnx, *entities):
         for entity in entities:
@@ -38,7 +46,7 @@ class SAEMRefHooksTC(CubicWebTC):
     def test_reset_md(self):
         """Ensure `resetMD` method above works."""
         with self.admin_access.repo_cnx() as cnx:
-            agent = create_agent(cnx, u'bob')
+            agent = testutils.authority_record(cnx, u'bob')
             cnx.commit()
             self.resetMD(cnx, agent)
             agent.cw_clear_all_caches()
@@ -46,11 +54,11 @@ class SAEMRefHooksTC(CubicWebTC):
 
     def test_sync_scheme_md(self):
         with self.admin_access.repo_cnx() as cnx:
-            scheme = cnx.create_entity('ConceptScheme', title=u'my thesaurus')
+            scheme = cnx.create_entity('ConceptScheme', title=u'my thesaurus',
+                                       ark_naa=testutils.naa(cnx))
             cnx.commit()
             self.resetMD(cnx, scheme)
-            c1 = cnx.create_entity('Concept', in_scheme=scheme)
-            cnx.create_entity('Label', label=u'concept 1', language_code=u'fr', label_of=c1)
+            c1 = scheme.add_concept(u'concept 1', language_code=u'fr')
             cnx.commit()
             self.assertMDNow(scheme)
             self.resetMD(cnx, scheme)
@@ -82,6 +90,8 @@ class SAEMRefHooksTC(CubicWebTC):
 
     def test_sync_profile_md(self):
         with self.admin_access.repo_cnx() as cnx:
+            testutils.seda_scheme(cnx, u'seda_access_restriction_code', u'preferred',
+                                  u'AR038')
             profile = testutils.setup_seda_profile(cnx)
             profile.cw_set(modification_date=YESTERDAY)
             cnx.commit()
@@ -92,13 +102,8 @@ class SAEMRefHooksTC(CubicWebTC):
             self.assertMDNow(profile)
             self.resetMD(cnx, profile)
             # edit link from root to an entity which is not part of the container
-            arch = cnx.create_entity('Agent', name=u'marcel archives',
-                                     agent_kind=cnx.find('AgentKind', name=u'person').one(),
-                                     archival_role=cnx.find('ArchivalRole', name='archival').one())
-            agent = cnx.create_entity('Agent', name=u'marcel',
-                                      agent_kind=cnx.find('AgentKind', name=u'person').one(),
-                                      archival_agent=arch,
-                                      archival_role=cnx.find('ArchivalRole', name='deposit').one())
+            agent = testutils.organization_unit(
+                cnx, u'marcel', archival_roles=['deposit'])
             profile.cw_set(seda_transferring_agent=agent)
             cnx.commit()
             self.assertMDNow(profile)
@@ -129,8 +134,8 @@ class SAEMRefHooksTC(CubicWebTC):
 
     def test_externaluri_to_concept(self):
         with self.admin_access.repo_cnx() as cnx:
-            # create some agent and related objects
-            agent = create_agent(cnx, u'bob')
+            # create some authority record and related objects
+            agent = testutils.authority_record(cnx, u'bob')
             address = cnx.create_entity('PostalAddress', street=u"1 av. de l'europe",
                                         postalcode=u'31400', city=u'Toulouse')
             place = cnx.create_entity('AgentPlace', place_address=address, place_agent=agent)
@@ -143,7 +148,7 @@ class SAEMRefHooksTC(CubicWebTC):
                                        reverse_equivalent_concept=[place, function, info])
             cnx.commit()
             # now insert a concept with the external uri as cwuri
-            scheme = cnx.create_entity('ConceptScheme')
+            scheme = cnx.create_entity('ConceptScheme', ark_naa=testutils.naa(cnx))
             concept = scheme.add_concept(u'some object', cwuri=u'http://someuri/someobject')
             cnx.commit()
             # ensure the external uri has been replaced by the concept and deleted
@@ -155,83 +160,89 @@ class SAEMRefHooksTC(CubicWebTC):
             self.assertEqual(info.equivalent_concept[0].eid, concept.eid)
             self.failIf(cnx.execute('Any X WHERE X eid %(x)s', {'x': exturi.eid}))
 
-    def test_externaluri_to_agent_subject(self):
+    def test_externaluri_to_authorityrecord_subject(self):
         with self.admin_access.repo_cnx() as cnx:
-            bob = create_agent(cnx, u'bob')
+            bob = testutils.authority_record(cnx, u'bob')
             exturi = cnx.create_entity('ExternalUri', cwuri=u'a/b/c', uri=u'a/b/c')
             arelation = cnx.create_entity('AssociationRelation',
                                           association_from=bob,
                                           association_to=exturi)
             cnx.commit()
-            alice = create_agent(cnx, u'alice', cwuri=u'a/b/c')
+            alice = testutils.authority_record(cnx, u'alice', cwuri=u'a/b/c')
             cnx.commit()
             arelation.cw_clear_all_caches()
             self.assertEqual(arelation.association_to[0], alice)
             self.failIf(cnx.execute('Any X WHERE X eid %(x)s', {'x': exturi.eid}))
 
-    def test_externaluri_to_agent_object(self):
+    def test_externaluri_to_authorityrecord_object(self):
         with self.admin_access.repo_cnx() as cnx:
-            bob = create_agent(cnx, u'bob')
+            bob = testutils.authority_record(cnx, u'bob')
             exturi = cnx.create_entity('ExternalUri', cwuri=u'a/b/c', uri=u'a/b/c')
             arelation = cnx.create_entity('AssociationRelation',
                                           association_from=exturi,
                                           association_to=bob)
             cnx.commit()
-            alice = create_agent(cnx, u'alice', cwuri=u'a/b/c')
+            alice = testutils.authority_record(cnx, u'alice', cwuri=u'a/b/c')
             cnx.commit()
             arelation.cw_clear_all_caches()
             self.assertEqual(arelation.association_from[0], alice)
             self.failIf(cnx.execute('Any X WHERE X eid %(x)s', {'x': exturi.eid}))
 
-    def test_ark_generation_agent(self):
+    def test_ark_generation_authorityrecord(self):
         with self.admin_access.repo_cnx() as cnx:
-            agent = create_agent(cnx, u'bob')
+            agent = testutils.authority_record(cnx, u'bob')
             cnx.commit()
-            self.assertEqual(agent.ark, 'saemref-test/%09d' % agent.eid)
-            self.assertEqual(agent.cwuri, 'ark:/saemref-test/%09d' % agent.eid)
-            agent = create_agent(cnx, u'john', ark=u'authority/123456')
+            self.assertEqual(agent.ark, '0/r%09d' % agent.eid)
+            self.assertEqual(agent.cwuri, 'ark:/0/r%09d' % agent.eid)
+            agent = testutils.authority_record(cnx, u'john', ark=u'authority/123456')
             cnx.commit()
             self.assertEqual(agent.ark, 'authority/123456')
             self.assertEqual(agent.cwuri, 'ark:/authority/123456')
-            agent = create_agent(cnx, u'alf', cwuri=u'http://someuri/someagent')
+            agent = testutils.authority_record(cnx, u'alf', cwuri=u'http://someuri/someagent')
             cnx.commit()
-            self.assertEqual(agent.ark, 'saemref-test/%09d' % agent.eid)
+            self.assertEqual(agent.ark, '0/r%09d' % agent.eid)
             self.assertEqual(agent.cwuri, 'http://someuri/someagent')
 
     def test_ark_generation_seda_profile(self):
         with self.admin_access.repo_cnx() as cnx:
             profile = testutils.setup_seda_profile(cnx)
-            self.assertEqual(profile.ark, 'saemref-test/{0:09d}'.format(profile.eid))
-            self.assertEqual(profile.cwuri, 'ark:/saemref-test/{0:09d}'.format(profile.eid))
-            # Profile with given ark
+            self.assertEqual(profile.ark, '0/p{0:09d}'.format(profile.eid))
+            self.assertEqual(profile.cwuri, 'ark:/0/p{0:09d}'.format(profile.eid))
+
+    def test_ark_generation_seda_profile_ark_given(self):
+        with self.admin_access.repo_cnx() as cnx:
             profile = testutils.setup_seda_profile(cnx, ark=u'authority/124')
             self.assertEqual(profile.ark, 'authority/124')
             self.assertEqual(profile.cwuri, 'ark:/authority/124')
-            # Imported profile with cwuri
-            profile = testutils.setup_seda_profile(cnx,
-                                                   cwuri=u'http://example.org/profile/125')
-            self.assertEqual(profile.ark, 'saemref-test/{0:09d}'.format(profile.eid))
+
+    def test_ark_generation_seda_profile_with_cwuri(self):
+        with self.admin_access.repo_cnx() as cnx:
+            profile = testutils.setup_seda_profile(
+                cnx, cwuri=u'http://example.org/profile/125')
+            self.assertEqual(profile.ark, '0/p{0:09d}'.format(profile.eid))
             self.assertEqual(profile.cwuri, 'http://example.org/profile/125')
 
     def test_ark_generation_concept(self):
         with self.admin_access.repo_cnx() as cnx:
-            scheme = cnx.create_entity('ConceptScheme')
+            scheme = cnx.create_entity('ConceptScheme', ark_naa=testutils.naa(cnx))
             concept = scheme.add_concept(u'some object')
             cnx.commit()
-            self.assertEqual(scheme.ark, 'saemref-test/%09d' % scheme.eid)
+            self.assertEqual(scheme.ark, '0/v%09d' % scheme.eid)
             self.assertEqual(scheme.cwuri,
-                             'ark:/saemref-test/%09d' % scheme.eid)
-            self.assertEqual(concept.ark, 'saemref-test/%09d' % concept.eid)
+                             'ark:/0/v%09d' % scheme.eid)
+            self.assertEqual(concept.ark, '0/c%09d' % concept.eid)
             self.assertEqual(concept.cwuri,
-                             'ark:/saemref-test/%09d' % concept.eid)
-            scheme = cnx.create_entity('ConceptScheme', cwuri=u'http://someuri/somescheme')
+                             'ark:/0/c%09d' % concept.eid)
+            scheme = cnx.create_entity('ConceptScheme', cwuri=u'http://someuri/somescheme',
+                                       ark_naa=testutils.naa(cnx))
             concept = scheme.add_concept(u'some object', cwuri=u'http://someuri/someconcept')
             cnx.commit()
-            self.assertEqual(scheme.ark, 'saemref-test/%09d' % scheme.eid)
+            self.assertEqual(scheme.ark, '0/v%09d' % scheme.eid)
             self.assertEqual(scheme.cwuri, 'http://someuri/somescheme')
-            self.assertEqual(concept.ark, 'saemref-test/%09d' % concept.eid)
+            self.assertEqual(concept.ark, '0/c%09d' % concept.eid)
             self.assertEqual(concept.cwuri, 'http://someuri/someconcept')
-            scheme = cnx.create_entity('ConceptScheme', cwuri=u'http://dcf/res/ark:/67717/Matiere')
+            scheme = cnx.create_entity('ConceptScheme', cwuri=u'http://dcf/res/ark:/67717/Matiere',
+                                       ark_naa=testutils.naa(cnx))
             concept = scheme.add_concept(u'some object', cwuri=u'http://dcf/res/ark:/67717/1234')
             cnx.commit()
             self.assertEqual(scheme.ark, '67717/Matiere')
@@ -239,69 +250,8 @@ class SAEMRefHooksTC(CubicWebTC):
             self.assertEqual(concept.ark, '67717/1234')
             self.assertEqual(concept.cwuri, 'http://dcf/res/ark:/67717/1234')
 
-    def test_check_archival_agent_relation(self):
-        with self.admin_access.repo_cnx() as cnx:
-            archival = cnx.find('ArchivalRole', name=u'archival').one()
-            archivist = cnx.create_entity('Agent',
-                                          name=u'archivist',
-                                          archival_role=archival,
-                                          agent_kind=cnx.find('AgentKind', name=u'person').one())
-            cnx.commit()
-            deposit = cnx.find('ArchivalRole', name=u'deposit').one()
-            with self.assertRaises(ValidationError) as cm:
-                cnx.create_entity('Agent', name=u'depositor', archival_role=deposit)
-                cnx.commit()
-            self.assertEqual(cm.exception.errors,
-                             {'archival_agent-subject':
-                              'this relation is mandatory on deposit agents'})
-            cnx.rollback()
-            # check it is not mandatory for another archival role
-            control = cnx.find('ArchivalRole', name=u'control').one()
-            cnx.create_entity('Agent', name=u'tata',
-                              archival_role=control,
-                              agent_kind=cnx.find('AgentKind', name=u'person').one())
-            # check no error if we give a archival_agent to depositor
-            depositor = cnx.create_entity('Agent', name=u'depositor',
-                                          archival_role=deposit,
-                                          archival_agent=archivist,
-                                          agent_kind=cnx.find('AgentKind', name=u'person').one())
-            # check one can change the archival agent of a deposit agent
-            new_archivist = cnx.create_entity('Agent', name=u'new_archivist',
-                                              archival_role=archival,
-                                              agent_kind=cnx.find('AgentKind',
-                                                                  name=u'person').one())
-            depositor.cw_set(archival_agent=new_archivist)
-            cnx.commit()
-            # check one can't remove the archival agent of a deposit agent
-            with self.assertRaises(ValidationError) as cm:
-                depositor.cw_set(archival_agent=None)
-                cnx.commit()
-            self.assertEqual(cm.exception.errors,
-                             {'archival_agent-subject':
-                              'this relation is mandatory on deposit agents'})
-            cnx.rollback()
-            # Check it is possible to delete an archival agent if it is not linked to any deposit
-            # agent
-            cnx.execute('DELETE Agent X WHERE X eid %s' % archivist.eid)
-            # check that if an archival agent is responsible of a deposit agent, it can't be deleted
-            with self.assertRaises(ValidationError) as cm:
-                cnx.execute('DELETE Agent X WHERE X eid %s' % new_archivist.eid)
-            self.assertEqual(cm.exception.errors,
-                             {'archival_role-subject':
-                              'this agent is the archival agent of other agents, therefore the '
-                              'role "archival" cannot be deleted'})
-            cnx.rollback()
-            # check that if an archival agent is responsible of a deposit agent, its "archival" role
-            # can't be deleted
-            with self.assertRaises(ValidationError) as cm:
-                new_archivist.cw_set(archival_role=None)
-            self.assertEqual(cm.exception.errors,
-                             {'archival_role-subject':
-                              'this agent is the archival agent of other agents, therefore the '
-                              'role "archival" cannot be deleted'})
 
-
-class ExtractArkTC(TestCase):
+class ExtractArkTC(unittest.TestCase):
 
     def test_ok(self):
         self.assertEqual(extract_ark('http://dcf/res/ark:/67717/1234'), '67717/1234')
@@ -315,29 +265,15 @@ class ExtractArkTC(TestCase):
         self.assertEqual(extract_ark('http://someuri/67717/1234'), None)
 
 
-class AgentUserTC(CubicWebTC):
-
-    def test_agent_added(self):
-        with self.admin_access.repo_cnx() as cnx:
-            bob = self.create_user(cnx, u'bob')
-            cnx.commit()
-            rset = cnx.find('Agent', name=u'bob')
-            self.assertEqual(len(rset), 1)
-            agent = rset.one()
-            self.assertEqual(agent.agent_kind[0].name, u'person')
-            self.assertEqual(agent.agent_user[0], bob)
-
-
 class EntityLifeCycleTC(CubicWebTC):
 
     def setup_database(self):
         with self.admin_access.repo_cnx() as cnx:
-            bob = self.create_user(cnx, u'bob')
-            cnx.commit()
-            self.bob_eid = bob.reverse_agent_user[0].eid
-            self.akind_eid = cnx.create_entity('AgentKind', name=u'in black').eid
-            cnx.create_entity('AgentKind', name=u'space beast')
+            bob = self.create_user(cnx, u'bob',
+                                   authority=cnx.find('Organization').one())
+            bob_agent = testutils.agent(cnx, u'bob', agent_user=bob)
             cnx.commit()
+            self.bob_eid = bob_agent.eid
 
     def _check_create(self, cnx, eid, msg):
         activity = cnx.find('Activity', type=u'create', used=eid).one()
@@ -352,104 +288,113 @@ class EntityLifeCycleTC(CubicWebTC):
         self.assertEqual(activity.generated[0].eid, eid)
         self.assertEqual(activity.description, msg)
 
-    def test_agent_create_update(self):
+    def test_authorityrecord_create_update(self):
         with self.new_access(u'bob').repo_cnx() as cnx:
-            agent = cnx.create_entity('Agent', name=u'Smith', agent_kind=self.akind_eid)
+            authorityrecord = testutils.authority_record(cnx, u'Smith')
             cnx.commit()
-            self.set_description('agent creation')
-            self._check_create(cnx, agent.eid, 'created agent')
-            agent.cw_set(isni=u'123', name=u'Adam')
+            self._check_create(cnx, authorityrecord.eid,
+                               'created authorityrecord')
+            authorityrecord.cw_set(isni=u'123', name=u'Adam')
             cnx.commit()
-            self.set_description('agent update')
-            self._check_modification(cnx, agent.eid, 'modified isni, name')
+            self._check_modification(cnx, authorityrecord.eid,
+                                     'modified isni, name')
 
-    def test_agent_related_change(self):
+    def test_authorityrecord_add_component(self):
         with self.new_access(u'bob').repo_cnx() as cnx:
-            agent = cnx.create_entity('Agent', name=u'Smith', agent_kind=self.akind_eid)
+            authorityrecord = testutils.authority_record(cnx, u'Smith')
             cnx.commit()
-            agent.cw_set(agent_kind=cnx.execute('AgentKind X WHERE X name "space beast"').one())
+            cnx.create_entity('AgentFunction', name=u'secret agent',
+                              function_agent=authorityrecord)
             cnx.commit()
-            self._check_modification(cnx, agent.eid, 'modified agent_kind')
+            self._check_modification(cnx, authorityrecord.eid,
+                                     'added function_agent_object')
 
-    def test_agent_add_component(self):
+    def test_authorityrecord_delete_component(self):
         with self.new_access(u'bob').repo_cnx() as cnx:
-            agent = cnx.create_entity('Agent', name=u'Smith', agent_kind=self.akind_eid)
+            authorityrecord = testutils.authority_record(cnx, u'Smith')
+            af = cnx.create_entity('AgentFunction', name=u'secret',
+                                   function_agent=authorityrecord)
             cnx.commit()
-            cnx.create_entity('PhoneNumber', number=u'99 88 77', reverse_phone_number=agent)
+            af.cw_delete()
             cnx.commit()
-            self._check_modification(cnx, agent.eid, 'added phone_number')
+            self._check_modification(cnx, authorityrecord.eid,
+                                     'removed function_agent_object')
 
-    def test_agent_delete_component(self):
+    def test_authorityrecord_update_component(self):
         with self.new_access(u'bob').repo_cnx() as cnx:
-            agent = cnx.create_entity('Agent', name=u'Smith', agent_kind=self.akind_eid)
-            pn = cnx.create_entity('PhoneNumber', number=u'99 88 77',
-                                   reverse_phone_number=agent)
+            authorityrecord = testutils.authority_record(cnx, u'Smith')
+            pn = cnx.create_entity('AgentFunction', name=u'secret agent',
+                                   function_agent=authorityrecord)
             cnx.commit()
-            pn.cw_delete()
+            pn.cw_set(name=u'boss of secret agents')
             cnx.commit()
-            self._check_modification(cnx, agent.eid, 'removed phone_number')
+            self._check_modification(cnx, authorityrecord.eid,
+                                     'modified function_agent_object')
 
-    def test_agent_update_component(self):
+    def test_authorityrecord_add_subcomponent(self):
         with self.new_access(u'bob').repo_cnx() as cnx:
-            agent = cnx.create_entity('Agent', name=u'Smith', agent_kind=self.akind_eid)
-            pn = cnx.create_entity('PhoneNumber', number=u'99 88 77', reverse_phone_number=agent)
-            cnx.commit()
-            pn.cw_set(number=u'11 22 33')
-            cnx.commit()
-            self._check_modification(cnx, agent.eid, 'modified phone_number')
-
-    def test_agent_add_subcomponent(self):
-        with self.new_access(u'bob').repo_cnx() as cnx:
-            agent = cnx.create_entity('Agent', name=u'Smith', agent_kind=self.akind_eid)
+            authorityrecord = testutils.authority_record(cnx, u'Smith')
             cnx.commit()
             address = cnx.create_entity('PostalAddress', street=u"1 av. de l'europe",
                                         postalcode=u'31400', city=u'Toulouse')
-            cnx.create_entity('AgentPlace', place_address=address, place_agent=agent)
+            cnx.create_entity('AgentPlace', place_address=address,
+                              place_agent=authorityrecord)
             cnx.commit()
-            self._check_modification(cnx, agent.eid, 'added place_address, place_agent_object')
+            self._check_modification(cnx, authorityrecord.eid,
+                                     'added place_address, place_agent_object')
 
-    def test_agent_delete_subcomponent(self):
+    def test_authorityrecord_delete_subcomponent(self):
         with self.new_access(u'bob').repo_cnx() as cnx:
-            agent = cnx.create_entity('Agent', name=u'Smith', agent_kind=self.akind_eid)
+            authorityrecord = testutils.authority_record(cnx, u'Smith')
             address = cnx.create_entity('PostalAddress', street=u"1 av. de l'europe",
                                         postalcode=u'31400', city=u'Toulouse')
-            cnx.create_entity('AgentPlace', place_address=address, place_agent=agent)
+            cnx.create_entity('AgentPlace', place_address=address,
+                              place_agent=authorityrecord)
             cnx.commit()
             address.cw_delete()
             cnx.commit()
-            self._check_modification(cnx, agent.eid, 'removed place_address')
+            self._check_modification(cnx, authorityrecord.eid, 'removed place_address')
 
-    def test_agent_update_subcomponent(self):
+    def test_authorityrecord_update_subcomponent(self):
         with self.new_access(u'bob').repo_cnx() as cnx:
-            agent = cnx.create_entity('Agent', name=u'Smith', agent_kind=self.akind_eid)
+            authorityrecord = testutils.authority_record(cnx, u'Smith')
             address = cnx.create_entity('PostalAddress', street=u"1 av. de l'europe",
                                         postalcode=u'31400', city=u'Toulouse')
-            cnx.create_entity('AgentPlace', place_address=address, place_agent=agent)
+            cnx.create_entity('AgentPlace', place_address=address,
+                              place_agent=authorityrecord)
             cnx.commit()
             address.cw_set(street=u"1 avenue de l'Europe")
             cnx.commit()
-            self._check_modification(cnx, agent.eid, 'modified place_address')
+            self._check_modification(cnx, authorityrecord.eid,
+                                     'modified place_address')
 
-    def test_agent_multi_modification(self):
+    def test_authorityrecord_multi_modification(self):
         with self.new_access(u'bob').repo_cnx() as cnx:
-            agent = cnx.create_entity('Agent', name=u'Smith', agent_kind=self.akind_eid)
-            pn = cnx.create_entity('PhoneNumber', number=u'99 88 77', reverse_phone_number=agent)
+            authorityrecord = testutils.authority_record(cnx, u'Smith')
+            f1 = cnx.create_entity('AgentFunction', name=u'special agent',
+                                   function_agent=authorityrecord)
+            gc = cnx.create_entity('GeneralContext', content=u'sunglasses',
+                                   general_context_of=authorityrecord)
             cnx.commit()
-            pn.cw_set(number=u'11 22 33')
+            gc.cw_set(content=u'stetson')
             address = cnx.create_entity('PostalAddress', street=u"1 av. de l'europe",
                                         postalcode=u'31400', city=u'Toulouse')
-            cnx.create_entity('AgentPlace', place_address=address, place_agent=agent)
-            agent.cw_set(isni=u'123', name=u'Adam',
-                         agent_kind=cnx.execute('AgentKind X WHERE X name "space beast"').one())
+            cnx.create_entity('AgentPlace', place_address=address,
+                              place_agent=authorityrecord)
+            f1.cw_delete()
+            cnx.create_entity('AgentFunction', name=u'mib',
+                              function_agent=authorityrecord)
+            authorityrecord.cw_set(isni=u'123', name=u'Adam')
             cnx.commit()
-            self._check_modification(cnx, agent.eid,
-                                     '* modified agent_kind, isni, name, phone_number\n'
-                                     '* added place_address, place_agent_object')
+            self._check_modification(
+                cnx, authorityrecord.eid,
+                '* modified function_agent_object, general_context_of_object, isni, name\n'
+                '* added place_address, place_agent_object')
 
-    def test_agent2agent_relation(self):
+    def test_authorityrecord2authorityrecord_relation(self):
         with self.new_access(u'bob').repo_cnx() as cnx:
-            k = cnx.create_entity('Agent', name=u'Agent K', agent_kind=self.akind_eid)
-            d = cnx.create_entity('Agent', name=u'Agent D', agent_kind=self.akind_eid)
+            k = testutils.authority_record(cnx, u'authorityrecord K')
+            d = testutils.authority_record(cnx, u'authorityrecord D')
             cnx.commit()
             cnx.create_entity('AssociationRelation', association_from=k, association_to=d)
             cnx.commit()
@@ -458,32 +403,37 @@ class EntityLifeCycleTC(CubicWebTC):
 
     def test_no_activity_generated(self):
         with self.admin_access.repo_cnx() as cnx:
-            agent = cnx.create_entity('Agent', name=u'Smith', agent_kind=self.akind_eid)
+            agent = cnx.create_entity('Agent', name=u'smith',
+                                      authority=cnx.find('Organization')[0][0])
             cnx.commit()
-            kind = cnx.execute('AgentKind X WHERE X name "in black"').one()
-            kind.cw_set(name=u'in white')
+            authorityrecord = testutils.authority_record(
+                cnx, u'Smith', reverse_authority_record=agent)
             cnx.commit()
-            self.failIf(cnx.find('Activity', type=u'modify', used=agent.eid))
+            agent.cw_set(name=u'jones')
+            cnx.commit()
+            self.failIf(cnx.find('Activity', type=u'modify',
+                                 used=authorityrecord.eid))
 
     def test_concept(self):
         with self.new_access(u'bob').repo_cnx() as cnx:
-            scheme = cnx.create_entity('ConceptScheme', title=u'my thesaurus')
+            scheme = cnx.create_entity('ConceptScheme', title=u'my thesaurus',
+                                       ark_naa=testutils.naa(cnx))
             cnx.commit()
-            self.set_description('concept scheme creation')
+            # concept scheme creation
             self._check_create(cnx, scheme.eid, 'created conceptscheme')
             concept = scheme.add_concept(u'hello')
             cnx.commit()
-            self.set_description('concept scheme add concept')
+            # concept scheme add concept
             self._check_modification(cnx, scheme.eid, 'added in_scheme_object')
-            self.set_description('concept creation')
+            # concept creation
             self._check_create(cnx, concept.eid, 'created concept')
             subconcept = concept.add_concept(u'goodbye')
             cnx.commit()
-            self.set_description('concept add concept')
+            # concept add concept
             self._check_modification(cnx, concept.eid, 'added broader_concept_object')
-            self.set_description('subconcept creation')
+            # subconcept creation
             self._check_create(cnx, subconcept.eid, 'created concept')
-            self.set_description('subconcept creation scheme')
+            # subconcept creation scheme
             rset = cnx.execute('Activity X ORDERBY X DESC WHERE X type "modify", X used %(x)s',
                                {'x': scheme.eid})
             self.assertEqual(len(rset), 2)
@@ -514,7 +464,8 @@ class SEDAProfileHooksTC(CubicWebTC):
             cnx.commit()
             profile.cw_clear_all_caches()
             self.assertEqual(workflow.state, 'published')
-            cloned = cnx.create_entity('SEDAProfile', title=u'Clone', seda_replace=profile)
+            cloned = cnx.create_entity('SEDAProfile', title=u'Clone', seda_replace=profile,
+                                       ark_naa=testutils.naa(cnx))
             cnx.commit()
             cloned.cw_adapt_to('IWorkflowable').fire_transition('publish')
             cnx.commit()
@@ -523,7 +474,7 @@ class SEDAProfileHooksTC(CubicWebTC):
 
     def test_profile_export_compat(self):
         with self.admin_access.client_cnx() as cnx:
-            profile = cnx.create_entity('SEDAProfile')
+            profile = cnx.create_entity('SEDAProfile', ark_naa=testutils.naa(cnx))
             cnx.commit()
             self.assertEqual(profile.support_seda_exports, '')
             ao = cnx.create_entity('ProfileArchiveObject',
@@ -539,5 +490,4 @@ class SEDAProfileHooksTC(CubicWebTC):
 
 
 if __name__ == '__main__':
-    from logilab.common.testlib import unittest_main
-    unittest_main()
+    unittest.main()
