@@ -1,4 +1,4 @@
-# copyright 2015 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# copyright 2015-2016 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 # contact http://www.logilab.fr -- mailto:contact@logilab.fr
 #
 # This program is free software: you can redistribute it and/or modify it under
@@ -17,16 +17,14 @@
 
 from datetime import datetime
 
-import pytz
+import unittest
 
-from logilab.common.testlib import TestCase, unittest_main
-
-from cubicweb import devtools
+from cubicweb import devtools  # noqa
 
 from cubes.saem_ref.site_cubicweb import print_tzdatetime_local
 
 
-class PrintTZDatetimeTC(TestCase):
+class PrintTZDatetimeTC(unittest.TestCase):
 
     def test(self):
         class req(object):
@@ -39,4 +37,4 @@ class PrintTZDatetimeTC(TestCase):
                          '2015/05/26 16:12')
 
 if __name__ == '__main__':
-    unittest_main()
+    unittest.main()
