@@ -21,17 +21,19 @@ from collections import defaultdict
 
 from logilab.mtconverter import xml_escape
 
-from cubicweb import tags, uilib, utils, Unauthorized
+from cubicweb import tags, uilib, utils
 from cubicweb.view import EntityView
 from cubicweb.predicates import (is_instance, score_entity, one_line_rset, multi_lines_rset,
                                  has_permission, match_form_params, has_related_entities,
-                                 is_in_state)
+                                 is_in_state, relation_possible)
 from cubicweb.web import (Redirect, httpcache, action, component, controller,
                           formwidgets as fw, formfields as ff)
-from cubicweb.web.views import (idownloadable, uicfg, baseviews, tableview, primary,
-                                ajaxcontroller, formrenderers, actions, editforms)
+from cubicweb.web.views import (ibreadcrumbs, idownloadable, uicfg, baseviews, basecomponents,
+                                tableview, primary,
+                                formrenderers, actions, editforms)
 
 from cubes.relationwidget import views as rwdg
+from cubes.compound.entities import copy_entity
 from cubes.compound.views import CloneAction
 
 from cubes.saem_ref.views import jqtree
@@ -42,6 +44,17 @@ from cubes.saem_ref.views.widgets import ConceptAutoCompleteWidget
 _ = unicode
 
 
+def selectable_uicfg(rtag, modname, select):
+    """Return a copy of uicfg `rtag` with `select` set.
+
+    Usually `modname` should be set to ``__name__`` from the caller.
+    """
+    copy = deepcopy(rtag)
+    copy.__module__ = modname
+    copy.__select__ = select
+    return copy
+
+
 # custom widget ####################################################################################
 
 class SEDAMetaFieldMixIn(object):
@@ -176,8 +189,6 @@ pvs.tag_attribute(('SEDAProfile', 'ark'), 'attributes')
 pvs.tag_object_of(('*', 'seda_keyword_scheme', '*'), 'hidden')
 pvs.tag_subject_of(('ConceptScheme', 'scheme_relation', '*'), 'hidden')
 afs.tag_subject_of(('ConceptScheme', 'scheme_relation', '*'), 'main', 'hidden')
-# related Agent
-pvs.tag_subject_of(('*', 'seda_transferring_agent', 'Agent'), 'hidden')
 
 # hide default relation components for 'seda_replace' rtype
 pvs.tag_subject_of(('*', 'seda_replace', '*'), 'hidden')
@@ -259,15 +270,16 @@ class AppraisalRuleValueView(EntityView):
 
     def entity_call(self, entity):
         _ = self._cw._
-        if entity.seda_appraisal_rule_duration:
-            duration = entity.seda_appraisal_rule_duration[0].value or _('<no value>')
+        if (entity.seda_appraisal_rule_duration and
+                entity.appraisal_rule_duration.seda_appraisal_rule_duration_value):
+            rule = entity.appraisal_rule_duration
+            duration = rule.seda_appraisal_rule_duration_value[0].view('saem.concept.alt')
         else:
             duration = _('<no value>')
-        self.w('<div><span>%s</span> %s</div>' % (_('seda_appraisal_rule_duration'),
-                                                  xml_escape(duration)))
+        self.w('<div><span>%s</span> %s</div>' % (_('seda_appraisal_rule_duration'), duration))
         if (entity.seda_appraisal_rule_code and
-                entity.seda_appraisal_rule_code[0].seda_appraisal_rule_code_value):
-            rule = entity.seda_appraisal_rule_code[0]
+                entity.appraisal_rule_code.seda_appraisal_rule_code_value):
+            rule = entity.appraisal_rule_code
             code = rule.seda_appraisal_rule_code_value[0].view('oneline')
         else:
             code = xml_escape(_('<no value>'))
@@ -322,7 +334,6 @@ rtype_concept_pref_label_choices = partial(rtype_concept_label_choices,
                                            label_kind=u'preferred_label')
 
 
-afs = uicfg.autoform_section
 affk = uicfg.autoform_field_kwargs
 aff = uicfg.autoform_field
 
@@ -369,7 +380,10 @@ afs.tag_subject_of(('SEDAAppraisalRule', 'seda_appraisal_rule_duration', '*'),
                    'main', 'inlined')
 afs.tag_subject_of(('SEDAAppraisalRuleCode', 'seda_appraisal_rule_code_value', '*'),
                    'main', 'attributes')
-affk.set_field_kwargs('SEDAAppraisalRuleDuration', 'value', widget=fw.TextInput({'size': 80}))
+afs.tag_subject_of(('SEDAAppraisalRuleDuration', 'seda_appraisal_rule_duration_value', '*'),
+                   'main', 'attributes')
+affk.tag_subject_of(('SEDAAppraisalRuleDuration', 'seda_appraisal_rule_duration_value', '*'),
+                    {'choices': rtype_concept_alt_label_choices})
 
 # simple link-to-concept attributes (for archive, archive object and document)
 for attr in ('seda_access_restriction_code',
@@ -457,9 +471,6 @@ for rtype in ('seda_access_restriction_code', 'seda_document_type_code', 'seda_f
     pvs.tag_object_of(('*', rtype, '*'), 'hidden')
     pvs.tag_object_of(('*', rtype + '_value', '*'), 'hidden')
 
-# hide related agents
-afs.tag_subject_of(('*', 'seda_transferring_agent', '*'), 'main', 'hidden')
-
 # finalize SEDAProfile form
 affk.set_field_kwargs('SEDAProfile', 'ark', widget=fw.TextInput({'size': 80}), required=False)
 affk.set_field_kwargs('SEDAProfile', 'title', widget=fw.TextInput({'size': 80}))
@@ -545,6 +556,31 @@ class ConceptAltLabelView(EntityView):
             xml_escape(title)))
 
 
+# navigation #######################################################################################
+
+class SEDAComponentsBreadcrumbsAdapter(ibreadcrumbs.IBreadCrumbsAdapter):
+    """Breadcrumbs adapter pointing to /sedalib route when entity has no
+    parent.
+    """
+    __select__ = (ibreadcrumbs.IBreadCrumbsAdapter.__select__
+                  & relation_possible('seda_parent', role='subject')
+                  & ~has_related_entities('seda_parent', role='subject'))
+
+    def breadcrumbs(self, *args, **kwargs):
+        return [(self._cw.build_url('sedalib'), self._cw._('SEDA components')),
+                self.entity]
+
+
+class SEDAComponentsBreadCrumbETypeVComponent(ibreadcrumbs.BreadCrumbEntityVComponent):
+    """For proper display of the breadcrumb in the SEDA components list"""
+    __select__ = (basecomponents.HeaderComponent.__select__
+                  & multi_lines_rset() & is_instance('ProfileArchiveObject', 'ProfileDocument'))
+
+    def render_breadcrumbs(self, w, contextentity, path):
+        w(u'<a href="%s">%s</a>' % (self._cw.build_url('sedalib'),
+                                    self._cw._('SEDA components')))
+
+
 # hack autoform_section rtags using the draft 'selectable' rtags feature ###########################
 
 def related_to_profile_archive(entity):
@@ -560,13 +596,30 @@ def related_to_profile_archive(entity):
         return 0  # XXX attempt to retrieve parent
     return int(parent.first_level)
 
-paac_afs = deepcopy(afs)
-paac_afs.__module__ = __name__
-paac_afs.__select__ = (is_instance('SEDAAccessRestrictionCode')
-                       & score_entity(related_to_profile_archive))
+
+paac_afs = selectable_uicfg(afs, __name__,
+                            (is_instance('SEDAAccessRestrictionCode')
+                             & score_entity(related_to_profile_archive)))
 paac_afs.tag_attribute(('SEDAAccessRestrictionCode', 'user_cardinality'), 'main', 'hidden')
 
 
+seda_clonable_afs = selectable_uicfg(afs, __name__,
+                                     (relation_possible('seda_clone_of')
+                                      & match_form_params(vid='copy')))
+
+for rtype in ('seda_description',
+              'seda_character_set_code',
+              'seda_file_type_code'):
+    seda_clonable_afs.tag_subject_of(('ProfileDocument', rtype, '*'),
+                                     'main', 'hidden')
+for rtype in ('seda_name',
+              'seda_access_restriction_code',
+              'seda_content_description',
+              'seda_appraisal_rule'):
+    seda_clonable_afs.tag_subject_of(('ProfileArchiveObject', rtype, '*'),
+                                     'main', 'hidden')
+
+
 # custom form renderers ############################################################################
 
 class SEDAValueEntityInlinedFormRenderer(formrenderers.EntityInlinedFormRenderer):
@@ -580,7 +633,7 @@ class SEDAValueEntityInlinedFormRenderer(formrenderers.EntityInlinedFormRenderer
         """Overriden from :class:`formrenderers.EntityInlinedFormRenderer` to skip the title"""
         if values['removejs']:
             w(u'<div class="iformTitle">')
-            values['removemsg'] = self._cw._('remove-inlined-entity-form')
+            values['removemsg'] = self._cw.__('remove-inlined-entity-form')
             w(u'[<a href="javascript: %(removejs)s;$.noop();">%(removemsg)s</a>]'
               % values)
             w(u'</div>')
@@ -610,7 +663,7 @@ class SEDACompoundEntityInlinedFormRenderer(formrenderers.EntityInlinedFormRende
             w(u'<div id="div-%(divid)s" class="formGroup">' % values)
         else:
             w(u'<div id="notice-%s" class="notice">%s</div>' % (
-                values['divid'], self._cw._('click on the box to cancel the deletion')))
+                values['divid'], self._cw.__('click on the box to cancel the deletion')))
         w(u'<div class="iformBody">')
 
 
@@ -619,6 +672,7 @@ class SEDACompoundEntityInlinedFormRenderer(formrenderers.EntityInlinedFormRende
 class SearchForRelatedConceptsView(rwdg.SearchForRelatedEntitiesView):
     __abstract__ = True
     rtype = None
+    has_creation_form = False
 
     def linkable_rset(self):
         """Return rset of entities to be displayed as possible values for the edited relation."""
@@ -679,18 +733,56 @@ class SelectConceptEntitiesTableView(rwdg.SelectEntitiesTableView):
 
 # tree box #########################################################################################
 
+def in_sedaprofile(entity):
+    """Return 1 if entity is a (possibly indirect) child of a SEDAProfile."""
+    while getattr(entity, 'seda_parent', None):
+        entity = entity.seda_parent[0]
+        if entity.cw_etype == 'SEDAProfile':
+            return 1
+    return 0
+
+
 class SEDAProfileTreeBox(component.EntityCtxComponent):
     """display a box containing the whole SEDA profile tree"""
     __regid__ = 'saem.seda.tree'
-    __select__ = is_instance(
-        'SEDAProfile', 'ProfileArchiveObject', 'ProfileDocument')
-    title = _('SEDA objects tree')
+    __select__ = one_line_rset() & (has_related_entities('seda_parent', role='subject')
+                                    | has_related_entities('seda_parent', role='object'))
+    title = _('SEDA profile objects tree')
     order = 10
 
     def render_body(self, w):
+        self._cw.add_css('cubes.jqtree.css')
         self.entity.view('jqtree.treeview', w=w)
 
 
+class ProfileArchiveObjectTreeBox(SEDAProfileTreeBox):
+    __select__ = SEDAProfileTreeBox.__select__ & ~score_entity(in_sedaprofile)
+    title = _('Profile archive object objects tree')
+
+
+class SEDAIJQTreeAdapter(jqtree.IJQTreeAdapter):
+    __select__ = jqtree.IJQTreeAdapter.__select__ & is_instance(
+        'SEDAProfile', 'ProfileArchiveObject', 'ProfileDocument')
+
+    def maybe_parent_of(self):
+        parents = []
+        rschema = self._cw.vreg.schema['seda_parent']
+        for subjtype, objtype in rschema.rdefs:
+            if objtype.type == self.entity.cw_etype:
+                parents.append(subjtype.type)
+        return parents
+
+    def maybe_child(self):
+        return 'seda_parent' in self.entity.e_schema.subjrels
+
+    def reparent(self, peid):
+        """Add a `seda_parent` relation between two entities."""
+        rset = self._cw.execute(
+            'SET C seda_parent P WHERE C eid %(c)s, P eid %(p)s',
+            {'c': self.entity.eid, 'p': peid})
+        return rset.rows
+
+
 class SEDAJsonTreeView(jqtree.JsonTreeView):
     __select__ = jqtree.JsonTreeView.__select__ & is_instance(
         'SEDAProfile', 'ProfileArchiveObject', 'ProfileDocument')
@@ -716,45 +808,6 @@ class JQTreeErrorItemLabelView(jqtree.JQTreeItemLabelView):
         self.w(tags.span(u'', title=title, klass='seda-export-error icon-attention'))
 
 
-@ajaxcontroller.ajaxfunc(output_type='json')
-def seda_movable(self, eid):
-    """Return True if entity with `eid` may be moved in a SEDAProfile tree.
-    """
-    etype = self._cw.describe(eid)[0]
-    schema = self._cw.vreg.schema
-    eschema = schema[etype]
-    return 'seda_parent' in eschema.subjrels
-
-
-@ajaxcontroller.ajaxfunc(output_type='json')
-def seda_may_set_parent(self, fromeid, toeid):
-    """Return True if a `seda_parent` relation maybe be added between entities
-    with `fromeid` and `toeid` eids.
-    """
-    frometype = self._cw.describe(fromeid)[0]
-    toetype = self._cw.describe(toeid)[0]
-    schema = self._cw.vreg.schema
-    try:
-        rdef = schema['seda_parent'].rdef(frometype, toetype)
-    except KeyError:
-        # No seda_parent relation between requested entity types.
-        return False
-    try:
-        rdef.check_perm(self._cw, 'add', toeid=toeid, fromeid=fromeid)
-    except Unauthorized:
-        return False
-    return True
-
-
-@ajaxcontroller.ajaxfunc(output_type='json')
-def seda_reparent(self, child_eid, parent_eid):
-    """Add a `seda_parent` relation between two entities.
-    """
-    self._cw.execute(
-        'SET C seda_parent P WHERE C eid %(c)s, P eid %(p)s',
-        {'c': child_eid, 'p': parent_eid})
-
-
 # life-cycle management ############################################################################
 
 # Hide copy action for SEDA profiles
@@ -860,7 +913,7 @@ class SEDAPrimaryView(primary.PrimaryView):
 
     def entity_call(self, entity, **kwargs):
         super(SEDAPrimaryView, self).entity_call(entity, **kwargs)
-        self.w(tags.div(id=_import_div_id(entity), style='display: none'))
+        rwdg.boostrap_dialog(self.w, self._cw._, _import_div_id(entity), u'')
 
 
 class SEDAImportAction(action.Action):
@@ -872,7 +925,11 @@ class SEDAImportAction(action.Action):
 
     category = 'moreactions'
     submenu = _('import menu')
-    etype = title = None  # to be specified in concrete classes
+    etype = None  # to be specified in concrete classes
+
+    @property
+    def title(self):
+        return self._cw._(self.etype).lower()
 
     def fill_menu(self, box, menu):
         # when there is only one item in the sub-menu, replace the sub-menu by
@@ -885,23 +942,23 @@ class SEDAImportAction(action.Action):
         root = entity.cw_adapt_to('ITree').root()
         relation = 'seda_clone_of:%s:subject' % self.etype
         search_url = self._cw.build_url('ajax', fname='view', vid='search_related_entities',
-                                        __modal=1, multiple='', relation=relation,
+                                        __modal=1, multiple='1', relation=relation,
                                         etype=root.cw_etype, target=root.eid)
         title = self._cw._('Search entity to import')
         return configure_relation_widget(self._cw, _import_div_id(entity), search_url, title,
-                                         False, uilib.js.saem.buildSedaImportValidate(entity.eid))
+                                         True, uilib.js.saem.buildSedaImportValidate(entity.eid))
 
 
 class SEDAImportProfileArchiveObjectAction(SEDAImportAction):
     __regid__ = 'saem_ref.seda.import.documentunit'
     __select__ = SEDAImportAction.__select__ & is_instance('SEDAProfile', 'ProfileArchiveObject')
-    etype = title = 'ProfileArchiveObject'
+    etype = 'ProfileArchiveObject'
 
 
 class SEDAImportProfileDocumentAction(SEDAImportAction):
     __regid__ = 'saem_ref.seda.import.attachment'
     __select__ = SEDAImportAction.__select__ & is_instance('ProfileArchiveObject')
-    etype = title = 'ProfileDocument'
+    etype = 'ProfileDocument'
 
 
 class SEDADoImportView(controller.Controller):
@@ -909,13 +966,16 @@ class SEDADoImportView(controller.Controller):
     __select__ = match_form_params('eid', 'cloned')
 
     def publish(self, rset=None):
-        entity = self._cw.entity_from_eid(int(self._cw.form['eid']))
-        cloned = self._cw.entity_from_eid(int(self._cw.form['cloned']))
-        self._cw.create_entity(cloned.cw_etype, seda_parent=entity, seda_clone_of=cloned)
-        msg = self._cw._('%(etype)s #%(eid)s has been imported') % {
-            'etype': self._cw._(cloned.cw_etype),
-            'eid': cloned.eid}
-        raise Redirect(entity.absolute_url(__message=msg))
+        parent = self._cw.entity_from_eid(int(self._cw.form['eid']))
+        clones = []
+        for cloned_eid in self._cw.form['cloned'].split(','):
+            original = self._cw.entity_from_eid(int(cloned_eid))
+            clones.append(copy_entity(original, seda_parent=parent,
+                                      seda_clone_of=original))
+        basemsg = (_('{0} has been imported') if len(clones) == 1 else
+                   _('{0} have been imported'))
+        msg = self._cw._(basemsg).format(u', '.join(clone.dc_title() for clone in clones))
+        raise Redirect(parent.absolute_url(__message=msg))
 
 
 class SearchForDocumentUnitToImportView(rwdg.SearchForRelatedEntitiesView):
@@ -1081,8 +1141,7 @@ class AddEntityComponent(component.CtxComponent):
     __select__ = (component.CtxComponent.__select__
                   & multi_lines_rset()
                   & has_permission('add')
-                  & is_instance('ProfileArchiveObject', 'ProfileDocument')
-                  & score_entity(lambda x: not x.seda_parent))
+                  & is_instance('ProfileArchiveObject', 'ProfileDocument'))
     context = 'navtop'
 
     def render_body(self, w):
@@ -1095,9 +1154,8 @@ class AddEntityComponent(component.CtxComponent):
 class SEDALibView(baseviews.SameETypeListView):
     __regid__ = 'saem_ref.sedalib'
     __select__ = (multi_lines_rset()
-                  & is_instance('ProfileArchiveObject', 'ProfileDocument')
-                  & score_entity(lambda x: not x.seda_parent))
+                  & is_instance('ProfileArchiveObject', 'ProfileDocument'))
 
     @property
     def title(self):
-        return self._cw._('SEDA components library')
+        return self._cw._('SEDA components')
